@@ -50,7 +50,6 @@
 #include <shibsp/handler/LogoutInitiator.h>
 #include <shibsp/handler/SessionInitiator.h>
 #include <xmltooling/logging.h>
-#include <xmltooling/util/DateTime.h>
 #include <xmltooling/util/NDC.h>
 #include <xmltooling/util/URLEncoder.h>
 #include <xmltooling/util/XMLHelper.h>
@@ -109,7 +108,7 @@ namespace {
         }
     };
 
-    MessageDecoder* ADFSDecoderFactory(const pair<const DOMElement*,const XMLCh*>& p)
+    MessageDecoder* ADFSDecoderFactory(const DOMElement* const &, bool)
     {
         return new ADFSDecoder();
     }
@@ -124,7 +123,7 @@ namespace {
     {
     public:
         ADFSSessionInitiator(const DOMElement* e, const char* appId)
-            : AbstractHandler(e, Category::getInstance(SHIBSP_LOGCAT".SessionInitiator.ADFS"), nullptr, &m_remapper), m_appId(appId), m_binding(WSFED_NS) {
+            : AbstractHandler(e, Category::getInstance(SHIBSP_LOGCAT ".SessionInitiator.ADFS"), nullptr, this), m_appId(appId), m_binding(WSFED_NS) {
             // If Location isn't set, defer address registration until the setParent call.
             pair<bool,const char*> loc = getString("Location");
             if (loc.first) {
@@ -178,8 +177,9 @@ namespace {
     {
         auto_ptr_XMLCh m_protocol;
     public:
-        ADFSConsumer(const DOMElement* e, const char* appId)
-            : shibsp::AssertionConsumerService(e, appId, Category::getInstance(SHIBSP_LOGCAT".SSO.ADFS")), m_protocol(WSFED_NS) {}
+        ADFSConsumer(const DOMElement* e, const char* appId, bool deprecationSupport)
+            : shibsp::AssertionConsumerService(e, appId, Category::getInstance(SHIBSP_LOGCAT ".SSO.ADFS"), nullptr, nullptr, deprecationSupport),
+                m_protocol(WSFED_NS) {}
         virtual ~ADFSConsumer() {}
 
 #ifndef SHIBSP_LITE
@@ -208,7 +208,7 @@ namespace {
     {
     public:
         ADFSLogoutInitiator(const DOMElement* e, const char* appId)
-                : AbstractHandler(e, Category::getInstance(SHIBSP_LOGCAT".LogoutInitiator.ADFS")), m_appId(appId), m_binding(WSFED_NS) {
+                : AbstractHandler(e, Category::getInstance(SHIBSP_LOGCAT ".LogoutInitiator.ADFS")), m_appId(appId), m_binding(WSFED_NS) {
             // If Location isn't set, defer address registration until the setParent call.
             pair<bool,const char*> loc = getString("Location");
             if (loc.first) {
@@ -247,8 +247,8 @@ namespace {
     class SHIBSP_DLLLOCAL ADFSLogout : public AbstractHandler, public LogoutHandler
     {
     public:
-        ADFSLogout(const DOMElement* e, const char* appId)
-                : AbstractHandler(e, Category::getInstance(SHIBSP_LOGCAT".Logout.ADFS")), m_login(e, appId) {
+        ADFSLogout(const DOMElement* e, const char* appId, bool deprecationSupport)
+                : AbstractHandler(e, Category::getInstance(SHIBSP_LOGCAT ".Logout.ADFS")), m_login(e, appId, deprecationSupport) {
             m_initiator = false;
 #ifndef SHIBSP_LITE
             m_preserve.push_back("wreply");
@@ -291,17 +291,17 @@ namespace {
     #pragma warning( pop )
 #endif
 
-    SessionInitiator* ADFSSessionInitiatorFactory(const pair<const DOMElement*,const char*>& p)
+    SessionInitiator* ADFSSessionInitiatorFactory(const pair<const DOMElement*,const char*>& p, bool)
     {
         return new ADFSSessionInitiator(p.first, p.second);
     }
 
-    Handler* ADFSLogoutFactory(const pair<const DOMElement*,const char*>& p)
+    Handler* ADFSLogoutFactory(const pair<const DOMElement*,const char*>& p, bool deprecationSupport)
     {
-        return new ADFSLogout(p.first, p.second);
+        return new ADFSLogout(p.first, p.second, deprecationSupport);
     }
 
-    Handler* ADFSLogoutInitiatorFactory(const pair<const DOMElement*,const char*>& p)
+    Handler* ADFSLogoutInitiatorFactory(const pair<const DOMElement*,const char*>& p, bool)
     {
         return new ADFSLogoutInitiator(p.first, p.second);
     }
@@ -368,7 +368,13 @@ pair<bool,long> ADFSSessionInitiator::run(SPRequest& request, string& entityID,
         recoverRelayState(app, request, request, target, false);
         app.limitRedirect(request, target.c_str());
 
-        acClass = getString("authnContextClassRef", request);
+        // Default is to allow externally supplied settings.
+        pair<bool,bool> externalInput = getBool("externalInput");
+        unsigned int settingMask = HANDLER_PROPERTY_MAP | HANDLER_PROPERTY_FIXED;
+        if (!externalInput.first || externalInput.second)
+            settingMask |= HANDLER_PROPERTY_REQUEST;
+
+        acClass = getString("authnContextClassRef", request, settingMask);
     }
     else {
         // Check for a hardwired target value in the map or handler.
@@ -433,7 +439,7 @@ pair<bool,long> ADFSSessionInitiator::run(SPRequest& request, string& entityID,
         in.addmember("authnContextClassRef").string(acClass.second);
 
     // Remote the processing.
-    out = request.getServiceProvider().getListenerService()->send(in);
+    out = send(request, in);
     return unwrap(request, out);
 }
 
@@ -541,8 +547,15 @@ pair<bool,long> ADFSSessionInitiator::doRequest(
     auto_ptr_char dest(ep->getLocation());
     const URLEncoder* urlenc = XMLToolingConfig::getConfig().getURLEncoder();
 
+    const PropertySet* relyingParty = app.getRelyingParty(entity.first);
+
     string req=string(dest.get()) + (strchr(dest.get(),'?') ? '&' : '?') + "wa=wsignin1.0&wreply=" + urlenc->encode(acsLocation) +
-        "&wct=" + urlenc->encode(timebuf) + "&wtrealm=" + urlenc->encode(app.getString("entityID").second);
+        "&wct=" + urlenc->encode(timebuf) + "&wtrealm=" + urlenc->encode(relyingParty->getString("entityID").second);
+    if (!authnContextClassRef) {
+        pair<bool,const char*> rpClass = relyingParty->getString("authnContextClassRef");
+        if (rpClass.first)
+            authnContextClassRef = rpClass.second;
+    }
     if (authnContextClassRef)
         req += "&wauth=" + urlenc->encode(authnContextClassRef);
     if (!relayState.empty())
@@ -566,7 +579,7 @@ XMLObject* ADFSDecoder::decode(string& relayState, const GenericRequest& generic
 #ifdef _DEBUG
     xmltooling::NDC ndc("decode");
 #endif
-    Category& log = Category::getInstance(SHIBSP_LOGCAT".MessageDecoder.ADFS");
+    Category& log = Category::getInstance(SHIBSP_LOGCAT ".MessageDecoder.ADFS");
 
     log.debug("validating input");
     const HTTPRequest* httpRequest=dynamic_cast<const HTTPRequest*>(&genericRequest);
@@ -859,7 +872,7 @@ pair<bool,long> ADFSLogoutInitiator::run(SPRequest& request, bool isHandler) con
         headers.push_back("User-Agent");
         DDF out,in = wrap(request, &headers);
         DDFJanitor jin(in), jout(out);
-        out=request.getServiceProvider().getListenerService()->send(in);
+        out = send(request, in);
         return unwrap(request, out);
     }
 }
@@ -906,8 +919,9 @@ void ADFSLogoutInitiator::receive(DDF& in, ostream& out)
         }
         else {
             m_log.error("no issuing entityID found in session");
+            time_t revocationExp = session->getExpiration();
             session->unlock();
-            app->getServiceProvider().getSessionCache()->remove(*app, *req, resp.get());
+            app->getServiceProvider().getSessionCache()->remove(*app, *req, resp.get(), revocationExp);
         }
     }
     out << ret;
@@ -932,9 +946,10 @@ pair<bool,long> ADFSLogoutInitiator::doRequest(
             application.getServiceProvider().getTransactionLog()->write(*logout_event);
         }
 #endif
+        time_t revocationExp = session->getExpiration();
         sessionLocker.assign();
         session = nullptr;
-        application.getServiceProvider().getSessionCache()->remove(application, httpRequest, &httpResponse);
+        application.getServiceProvider().getSessionCache()->remove(application, httpRequest, &httpResponse, revocationExp);
         return sendLogoutPage(application, httpRequest, httpResponse, "partial");
     }
 
@@ -996,16 +1011,17 @@ pair<bool,long> ADFSLogoutInitiator::doRequest(
         ret.first = true;
 
         if (session) {
+            time_t revocationExp = session->getExpiration();
             sessionLocker.assign();
             session = nullptr;
-            application.getServiceProvider().getSessionCache()->remove(application, httpRequest, &httpResponse);
+            application.getServiceProvider().getSessionCache()->remove(application, httpRequest, &httpResponse, revocationExp);
         }
     }
-    catch (MetadataException& mex) {
+    catch (const MetadataException& mex) {
         // Less noise for IdPs that don't support logout
         m_log.info("unable to issue ADFS logout request: %s", mex.what());
     }
-    catch (std::exception& ex) {
+    catch (const std::exception& ex) {
         m_log.error("error issuing ADFS logout request: %s", ex.what());
     }
 
@@ -1030,7 +1046,7 @@ pair<bool,long> ADFSLogout::run(SPRequest& request, bool isHandler) const
         if (!strcmp(param, "wsignin1.0"))
             return m_login.run(request, isHandler);
         else if (strcmp(param, "wsignout1.0") && strcmp(param, "wsignoutcleanup1.0"))
-            throw FatalProfileException("Unsupported WS-Federation action paremeter ($1).", params(1, param));
+            throw FatalProfileException("Unsupported WS-Federation action parameter ($1).", params(1, param));
     }
     else if (strcmp(request.getMethod(),"GET") || !request.getParameter("notifying"))
         throw FatalProfileException("Unsupported request to ADFS protocol endpoint.");
@@ -1058,7 +1074,7 @@ pair<bool,long> ADFSLogout::run(SPRequest& request, bool isHandler) const
         try {
             app.getServiceProvider().getSessionCache()->remove(app, request, &request);
         }
-        catch (std::exception& ex) {
+        catch (const std::exception& ex) {
             m_log.error("error removing session (%s): %s", session_id.c_str(), ex.what());
         }
     }
