@@ -145,7 +145,7 @@ namespace {
         bool m_buildMap;
     };
 
-    StorageService* MemcacheStorageServiceFactory(const DOMElement* const & e) {
+    StorageService* MemcacheStorageServiceFactory(const DOMElement* const & e, bool) {
         return new MemcacheStorageService(e);
     }
 };
@@ -192,7 +192,7 @@ MemcacheBase::MemcacheBase(const DOMElement* e)
         memcached_free(memc);
         throw XMLToolingException("Memcache StorageService requires Hosts element in configuration.");
     }
-    auto_ptr_char h(e->getTextContent());
+    auto_ptr_char h(XMLHelper::getTextContent(e));
     m_log.debug("INIT: GOT Hosts: %s", h.get());
     memcached_server_st* servers;
     servers = memcached_servers_parse(const_cast<char*>(h.get()));
@@ -429,7 +429,7 @@ bool MemcacheStorageService::createString(const char* context, const char* key,
 {
     m_log.debug("createString ctx: %s - key: %s", context, key);
 
-    string final_key = string(context) + ":" + string(key);
+    string final_key = string(context) + ':' + string(key);
 
     mc_record rec(value, expiration);
     string final_value;
@@ -496,20 +496,19 @@ int MemcacheStorageService::readString(const char* context, const char* key, str
     if (!found)
         return 0;
 
+    mc_record rec;
+    if (pexpiration || pvalue)
+        deserialize(value, rec);
+    
+    if (pexpiration)
+        *pexpiration = rec.expiration;
+
     if (version && rec_version <= (uint32_t)version)
         return version;
 
-    if (pexpiration || pvalue) {
-        mc_record rec;
-        deserialize(value, rec);
-    
-        if (pexpiration)
-            *pexpiration = rec.expiration;
-    
-        if (pvalue)
-            *pvalue = rec.value;
-    }
-  
+    if (pvalue)
+        *pvalue = rec.value;
+
     return rec_version;
 }
 
@@ -535,7 +534,7 @@ int MemcacheStorageService::updateString(const char* context, const char* key, c
     }
 
     // Proceding with update
-    string final_key = string(context) + ":" + string(key);
+    string final_key = string(context) + ':' + string(key);
     mc_record rec(value, final_exp);
     string final_value;
     serialize(rec, final_value);
@@ -548,7 +547,7 @@ bool MemcacheStorageService::deleteString(const char* context, const char* key)
 {
     m_log.debug("deleteString ctx: %s - key: %s", context, key);
   
-    string final_key = string(context) + ":" + string(key);
+    string final_key = string(context) + ':' + string(key);
 
     // Not updating context map, if there is one. There is no need.
     return deleteMemcache(final_key.c_str(), 0);
@@ -612,7 +611,7 @@ void MemcacheStorageService::deleteContext(const char* context)
     
         m_log.debug("Iterating retrieved session map...");
         for (list<string>::const_iterator iter = contents.begin(); iter != contents.end(); ++iter) {
-            string final_key = map_name + *iter;
+            string final_key = map_name + ':' + *iter;
             deleteMemcache(final_key.c_str(), 0);
         }
     
