@@ -38,6 +38,7 @@
 #include <fstream>
 #include <sstream>
 #include <boost/algorithm/string.hpp>
+#include <boost/lexical_cast.hpp>
 #ifndef SHIBSP_LITE
 # include <saml/exceptions.h>
 # include <saml/saml2/metadata/MetadataProvider.h>
@@ -85,7 +86,7 @@ namespace shibsp {
             if (mayRedirect)
                 redirectErrors = settings.first->getString("redirectErrors");
         }
-        catch (exception& ex) {
+        catch (const exception& ex) {
             log.error(ex.what());
         }
 
@@ -117,7 +118,7 @@ namespace shibsp {
         string fname;
         if (!pathname.first) {
             if (!accesserror) {
-                fname = string(page) + "Error.html";
+                fname = string(mderror ? "metadata" : page) + "Error.html";
                 pathname.second = fname.c_str();
             }
         }
@@ -152,15 +153,120 @@ namespace shibsp {
         app.clearHeader(request, "Shib-Cookie-Name", "HTTP_SHIB_COOKIE_NAME");
         app.clearHeader(request, "Shib-Session-ID", "HTTP_SHIB_SESSION_ID");
         app.clearHeader(request, "Shib-Session-Index", "HTTP_SHIB_SESSION_INDEX");
+        app.clearHeader(request, "Shib-Session-Expires", "HTTP_SHIB_SESSION_EXPIRES");
+        app.clearHeader(request, "Shib-Session-Inactivity", "HTTP_SHIB_SESSION_INACTIVITY");
         app.clearHeader(request, "Shib-Identity-Provider", "HTTP_SHIB_IDENTITY_PROVIDER");
         app.clearHeader(request, "Shib-Authentication-Method", "HTTP_SHIB_AUTHENTICATION_METHOD");
         app.clearHeader(request, "Shib-Authentication-Instant", "HTTP_SHIB_AUTHENTICATION_INSTANT");
         app.clearHeader(request, "Shib-AuthnContext-Class", "HTTP_SHIB_AUTHNCONTEXT_CLASS");
         app.clearHeader(request, "Shib-AuthnContext-Decl", "HTTP_SHIB_AUTHNCONTEXT_DECL");
         app.clearHeader(request, "Shib-Assertion-Count", "HTTP_SHIB_ASSERTION_COUNT");
+        app.clearHeader(request, "Shib-Handler", "HTTP_SHIB_HANDLER");
         app.clearAttributeHeaders(request);
         request.clearHeader("REMOTE_USER", "HTTP_REMOTE_USER");
     }
+
+    void SHIBSP_DLLLOCAL exportAttributes(SPRequest& request, const Session* session, RequestMapper::Settings settings) {
+
+        pair<bool,const char*> enc = settings.first->getString("encoding");
+        if (enc.first && strcmp(enc.second, "URL"))
+            throw ConfigurationException("Unsupported value for 'encoding' content setting ($1).", params(1,enc.second));
+
+        const URLEncoder* encoder = XMLToolingConfig::getConfig().getURLEncoder();
+
+        pair<bool,bool> exportDups = settings.first->getBool("exportDuplicateValues");
+        const multimap<string,const Attribute*>& attributes = session->getIndexedAttributes();
+
+        // Default export strategy will include duplicates.
+        if (!exportDups.first || exportDups.second) {
+            for (multimap<string,const Attribute*>::const_iterator a = attributes.begin(); a != attributes.end(); ++a) {
+                if (a->second->isInternal())
+                    continue;
+                string header(request.getApplication().getSecureHeader(request, a->first.c_str()));
+                const vector<string>& vals = a->second->getSerializedValues();
+                for (vector<string>::const_iterator v = vals.begin(); v != vals.end(); ++v) {
+                    if (!header.empty())
+                        header += ';';
+                    if (enc.first) {
+                        // If URL-encoding, any semicolons will get escaped anyway.
+                        header += encoder->encode(v->c_str());
+                    }
+                    else {
+                        string::size_type pos = v->find_first_of(';', string::size_type(0));
+                        if (pos != string::npos) {
+                            string value(*v);
+                            for (; pos != string::npos; pos = value.find_first_of(';', pos)) {
+                                value.insert(pos, "\\");
+                                pos += 2;
+                            }
+                            header += value;
+                        }
+                        else {
+                            header += (*v);
+                        }
+                    }
+                }
+                request.getApplication().setHeader(request, a->first.c_str(), header.c_str());
+            }
+        }
+        else {
+            // Capture values in a map of sets to check for duplicates on the fly.
+            map< string,set<string> > valueMap;
+            for (multimap<string,const Attribute*>::const_iterator a = attributes.begin(); a != attributes.end(); ++a) {
+                if (a->second->isInternal())
+                    continue;
+                const vector<string>& vals = a->second->getSerializedValues();
+                valueMap[a->first].insert(vals.begin(), vals.end());
+            }
+
+            // Export the mapped sets to the headers.
+            for (map< string,set<string> >::const_iterator deduped = valueMap.begin(); deduped != valueMap.end(); ++deduped) {
+                string header;
+                for (set<string>::const_iterator v = deduped->second.begin(); v != deduped->second.end(); ++v) {
+                    if (!header.empty())
+                        header += ';';
+                    if (enc.first) {
+                        // If URL-encoding, any semicolons will get escaped anyway.
+                        header += encoder->encode(v->c_str());
+                    }
+                    else {
+                        string::size_type pos = v->find_first_of(';', string::size_type(0));
+                        if (pos != string::npos) {
+                            string value(*v);
+                            for (; pos != string::npos; pos = value.find_first_of(';', pos)) {
+                                value.insert(pos, "\\");
+                                pos += 2;
+                            }
+                            header += value;
+                        }
+                        else {
+                            header += (*v);
+                        }
+                    }
+                }
+                request.getApplication().setHeader(request, deduped->first.c_str(), header.c_str());
+            }
+        }
+
+        // Check for REMOTE_USER.
+        bool remoteUserSet = false;
+        const vector<string>& rmids = request.getApplication().getRemoteUserAttributeIds();
+        for (vector<string>::const_iterator rmid = rmids.begin(); !remoteUserSet && rmid != rmids.end(); ++rmid) {
+            pair<multimap<string,const Attribute*>::const_iterator,multimap<string,const Attribute*>::const_iterator> matches =
+                attributes.equal_range(*rmid);
+            for (; matches.first != matches.second; ++matches.first) {
+                const vector<string>& vals = matches.first->second->getSerializedValues();
+                if (!vals.empty()) {
+                    if (enc.first)
+                        request.setRemoteUser(encoder->encode(vals.front().c_str()).c_str());
+                    else
+                        request.setRemoteUser(vals.front().c_str());
+                    remoteUserSet = true;
+                    break;
+                }
+            }
+        }
+    }
 };
 
 void SHIBSP_API shibsp::registerServiceProviders()
@@ -177,46 +283,6 @@ ServiceProvider::~ServiceProvider()
 {
 }
 
-#ifndef SHIBSP_LITE
-SecurityPolicyProvider* ServiceProvider::getSecurityPolicyProvider(bool required) const
-{
-    if (required)
-        throw ConfigurationException("No SecurityPolicyProvider available.");
-    return nullptr;
-}
-#endif
-
-Remoted* ServiceProvider::regListener(const char* address, Remoted* listener)
-{
-    Remoted* ret = nullptr;
-    map<string,Remoted*>::const_iterator i = m_listenerMap.find(address);
-    if (i != m_listenerMap.end())
-        ret = i->second;
-    m_listenerMap[address] = listener;
-    Category::getInstance(SHIBSP_LOGCAT ".ServiceProvider").info("registered remoted message endpoint (%s)",address);
-    return ret;
-}
-
-bool ServiceProvider::unregListener(const char* address, Remoted* current, Remoted* restore)
-{
-    map<string,Remoted*>::const_iterator i = m_listenerMap.find(address);
-    if (i != m_listenerMap.end() && i->second == current) {
-        if (restore)
-            m_listenerMap[address] = restore;
-        else
-            m_listenerMap.erase(address);
-        Category::getInstance(SHIBSP_LOGCAT ".ServiceProvider").info("unregistered remoted message endpoint (%s)",address);
-        return true;
-    }
-    return false;
-}
-
-Remoted* ServiceProvider::lookupListener(const char *address) const
-{
-    map<string,Remoted*>::const_iterator i = m_listenerMap.find(address);
-    return (i == m_listenerMap.end()) ? nullptr : i->second;
-}
-
 pair<bool,long> ServiceProvider::doAuthentication(SPRequest& request, bool handler) const
 {
 #ifdef _DEBUG
@@ -231,6 +297,9 @@ pair<bool,long> ServiceProvider::doAuthentication(SPRequest& request, bool handl
         RequestMapper::Settings settings = request.getRequestSettings();
         app = &(request.getApplication());
 
+        string appid(string("[") + app->getId() + "]");
+        xmltooling::NDC ndc(appid.c_str());
+
         // If not SSL, check to see if we should block or redirect it.
         if (!request.isSecure()) {
             pair<bool,const char*> redirectToSSL = settings.first->getString("redirectToSSL");
@@ -289,10 +358,10 @@ pair<bool,long> ServiceProvider::doAuthentication(SPRequest& request, bool handl
         try {
             session = request.getSession(true, false, false);   // don't cache it
         }
-        catch (exception& e) {
+        catch (const exception& e) {
             log.warn("error during session lookup: %s", e.what());
             // If it's not a retryable session failure, we throw to the outer handler for reporting.
-            if (dynamic_cast<opensaml::RetryableProfileException*>(&e) == nullptr)
+            if (dynamic_cast<const opensaml::RetryableProfileException*>(&e) == nullptr)
                 throw;
         }
 
@@ -320,11 +389,14 @@ pair<bool,long> ServiceProvider::doAuthentication(SPRequest& request, bool handl
                     return make_pair(true, request.sendRedirect(loc.c_str()));
                 }
             }
+            app->setHeader(request, "Shib-Handler", handlerURL);
         }
         else {
             // No session.  Maybe that's acceptable?
-            if ((!requireSession.first || !requireSession.second) && !requireSessionWith.first)
+            if ((!requireSession.first || !requireSession.second) && !requireSessionWith.first) {
+                app->setHeader(request, "Shib-Handler", handlerURL);
                 return make_pair(true, request.returnOK());
+            }
 
             // No session, but we require one. Initiate a new session using the indicated method.
             const SessionInitiator* initiator=nullptr;
@@ -332,7 +404,7 @@ pair<bool,long> ServiceProvider::doAuthentication(SPRequest& request, bool handl
                 initiator=app->getSessionInitiatorById(requireSessionWith.second);
                 if (!initiator) {
                     throw ConfigurationException(
-                        "No session initiator found with id ($1), check requireSessionWith command.", params(1, requireSessionWith.second)
+                        "No session initiator found with id ($1), check requireSessionWith setting.", params(1, requireSessionWith.second)
                         );
                 }
             }
@@ -344,10 +416,17 @@ pair<bool,long> ServiceProvider::doAuthentication(SPRequest& request, bool handl
 
             // Dispatch to SessionInitiator. This MUST handle the request, or we want to fail here.
             // Used to fall through into doExport, but this is a cleaner exit path.
-            pair<bool,long> ret = initiator->run(request, false);
-            if (ret.first)
-                return ret;
-            throw ConfigurationException("Session initiator did not handle request for a new session, check configuration.");
+            try {
+                pair<bool, long> ret = initiator->run(request, false);
+                if (ret.first)
+                    return ret;
+                throw ConfigurationException("Session initiator did not handle request for a new session, check configuration.");
+            }
+            catch (XMLToolingException& ex) {
+                if (!ex.getProperty("eventType") && initiator->getEventType())
+                    ex.addProperty("eventType", initiator->getEventType());
+                throw;
+            }
         }
 
         request.setAuthType(authType.second);
@@ -357,7 +436,7 @@ pair<bool,long> ServiceProvider::doAuthentication(SPRequest& request, bool handl
         log.debug("doAuthentication succeeded");
         return make_pair(false,0L);
     }
-    catch (exception& e) {
+    catch (const exception& e) {
         request.log(SPRequest::SPError, e.what());
         TemplateParameters tp(&e);
         tp.m_map["requestURL"] = targetURL.substr(0,targetURL.find('?'));
@@ -381,6 +460,9 @@ pair<bool,long> ServiceProvider::doAuthorization(SPRequest& request) const
         RequestMapper::Settings settings = request.getRequestSettings();
         app = &(request.getApplication());
 
+        string appid(string("[") + app->getId() + "]");
+        xmltooling::NDC ndc(appid.c_str());
+
         // Three settings dictate how to proceed.
         pair<bool,const char*> authType = settings.first->getString("authType");
         pair<bool,bool> requireSession = settings.first->getBool("requireSession");
@@ -400,7 +482,7 @@ pair<bool,long> ServiceProvider::doAuthorization(SPRequest& request) const
                 if (session)
                     slocker.assign(session, false); // assign to lock popper
             }
-            catch (exception& e) {
+            catch (const exception& e) {
                 log.warn("unable to obtain session to pass to access control provider: %s", e.what());
             }
 
@@ -427,7 +509,7 @@ pair<bool,long> ServiceProvider::doAuthorization(SPRequest& request) const
             return make_pair(true, request.returnDecline());
         }
     }
-    catch (exception& e) {
+    catch (const exception& e) {
         request.log(SPRequest::SPError, e.what());
         TemplateParameters tp(&e, nullptr, session);
         tp.m_map["requestURL"] = targetURL.substr(0,targetURL.find('?'));
@@ -451,12 +533,15 @@ pair<bool,long> ServiceProvider::doExport(SPRequest& request, bool requireSessio
         RequestMapper::Settings settings = request.getRequestSettings();
         app = &(request.getApplication());
 
+        string appid(string("[") + app->getId() + "]");
+        xmltooling::NDC ndc(appid.c_str());
+
         try {
             session = request.getSession(false, false, false);  // ignore timeout and do not cache
             if (session)
                 slocker.assign(session, false); // assign to lock popper
         }
-        catch (exception& e) {
+        catch (const exception& e) {
             log.warn("unable to obtain session to export to request: %s", e.what());
         	// If we have to have a session, then this is a fatal error.
         	if (requireSession)
@@ -471,15 +556,11 @@ pair<bool,long> ServiceProvider::doExport(SPRequest& request, bool requireSessio
         		return make_pair(false, 0L);	// just bail silently
         }
 
-		pair<bool,const char*> enc = settings.first->getString("encoding");
-		if (enc.first && strcmp(enc.second, "URL"))
-			throw ConfigurationException("Unsupported value for 'encoding' content setting ($1).", params(1,enc.second));
-
-        const URLEncoder* encoder = XMLToolingConfig::getConfig().getURLEncoder();
-
         app->setHeader(request, "Shib-Application-ID", app->getId());
         app->setHeader(request, "Shib-Session-ID", session->getID());
 
+        const PropertySet* sessionProps = app->getPropertySet("Sessions");
+
         // Check for export of "standard" variables.
         // A 3.0 release would switch this default to false and rely solely on the
         // Assertion extractor plugin and ship out of the box with the same defaults.
@@ -502,6 +583,12 @@ pair<bool,long> ServiceProvider::doExport(SPRequest& request, bool requireSessio
             hval = session->getSessionIndex();
             if (hval)
                 app->setHeader(request, "Shib-Session-Index", hval);
+
+            app->setHeader(request, "Shib-Session-Expires", boost::lexical_cast<string>(session->getExpiration()).c_str());
+            pair<bool,unsigned int> timeout = sessionProps ? sessionProps->getUnsignedInt("timeout") : pair<bool,unsigned int>(false, 0);
+            if (timeout.first && timeout.second > 0) {
+                app->setHeader(request, "Shib-Session-Inactivity", boost::lexical_cast<string>(session->getLastAccess() + timeout.second).c_str());
+            }
         }
 
         // Check for export of algorithmically-derived portion of cookie names.
@@ -514,8 +601,7 @@ pair<bool,long> ServiceProvider::doExport(SPRequest& request, bool requireSessio
         // Maybe export the assertion keys.
         pair<bool,bool> exp = settings.first->getBool("exportAssertion");
         if (exp.first && exp.second) {
-            const PropertySet* sessions = app->getPropertySet("Sessions");
-            pair<bool,const char*> exportLocation = sessions ? sessions->getString("exportLocation") : pair<bool,const char*>(false,nullptr);
+            pair<bool,const char*> exportLocation = sessionProps ? sessionProps->getString("exportLocation") : pair<bool,const char*>(false,nullptr);
             if (!exportLocation.first)
                 log.warn("can't export assertions without an exportLocation Sessions property");
             else {
@@ -532,7 +618,7 @@ pair<bool,long> ServiceProvider::doExport(SPRequest& request, bool requireSessio
                     count++;
                     *(exportName.rbegin()) = '0' + (count%10);
                     *(++exportName.rbegin()) = '0' + (count/10);
-                    string fullURL = baseURL + encoder->encode(*tokenids);
+                    string fullURL = baseURL + XMLToolingConfig::getConfig().getURLEncoder()->encode(*tokenids);
                     app->setHeader(request, exportName.c_str(), fullURL.c_str());
                 }
                 app->setHeader(request, "Shib-Assertion-Count", exportName.c_str() + 15);
@@ -540,59 +626,11 @@ pair<bool,long> ServiceProvider::doExport(SPRequest& request, bool requireSessio
         }
 
         // Export the attributes.
-        const multimap<string,const Attribute*>& attributes = session->getIndexedAttributes();
-        for (multimap<string,const Attribute*>::const_iterator a = attributes.begin(); a != attributes.end(); ++a) {
-            if (a->second->isInternal())
-                continue;
-            string header(app->getSecureHeader(request, a->first.c_str()));
-            const vector<string>& vals = a->second->getSerializedValues();
-            for (vector<string>::const_iterator v = vals.begin(); v != vals.end(); ++v) {
-                if (!header.empty())
-                    header += ";";
-				if (enc.first) {
-					// If URL-encoding, any semicolons will get escaped anyway.
-					header += encoder->encode(v->c_str());
-				}
-				else {
-					string::size_type pos = v->find_first_of(';', string::size_type(0));
-					if (pos != string::npos) {
-						string value(*v);
-						for (; pos != string::npos; pos = value.find_first_of(';', pos)) {
-							value.insert(pos, "\\");
-							pos += 2;
-						}
-						header += value;
-					}
-					else {
-						header += (*v);
-					}
-				}
-            }
-            app->setHeader(request, a->first.c_str(), header.c_str());
-        }
-
-        // Check for REMOTE_USER.
-        bool remoteUserSet = false;
-        const vector<string>& rmids = app->getRemoteUserAttributeIds();
-        for (vector<string>::const_iterator rmid = rmids.begin(); !remoteUserSet && rmid != rmids.end(); ++rmid) {
-            pair<multimap<string,const Attribute*>::const_iterator,multimap<string,const Attribute*>::const_iterator> matches =
-                attributes.equal_range(*rmid);
-            for (; matches.first != matches.second; ++matches.first) {
-                const vector<string>& vals = matches.first->second->getSerializedValues();
-                if (!vals.empty()) {
-					if (enc.first)
-						request.setRemoteUser(encoder->encode(vals.front().c_str()).c_str());
-					else
-						request.setRemoteUser(vals.front().c_str());
-                    remoteUserSet = true;
-                    break;
-                }
-            }
-        }
+        exportAttributes(request, session, settings);
 
         return make_pair(false,0L);
     }
-    catch (exception& e) {
+    catch (const exception& e) {
         request.log(SPRequest::SPError, e.what());
         TemplateParameters tp(&e, nullptr, session);
         tp.m_map["requestURL"] = targetURL.substr(0,targetURL.find('?'));
@@ -614,6 +652,9 @@ pair<bool,long> ServiceProvider::doHandler(SPRequest& request) const
         RequestMapper::Settings settings = request.getRequestSettings();
         app = &(request.getApplication());
 
+        string appid(string("[") + app->getId() + "]");
+        xmltooling::NDC ndc(appid.c_str());
+
         // If not SSL, check to see if we should block or redirect it.
         if (!request.isSecure()) {
             pair<bool,const char*> redirectToSSL = settings.first->getString("redirectToSSL");
@@ -664,21 +705,26 @@ pair<bool,long> ServiceProvider::doHandler(SPRequest& request) const
         if (!handler)
             throw ConfigurationException("Shibboleth handler invoked at an unconfigured location.");
 
-        pair<bool,long> hret = handler->run(request);
-
-        // Did the handler run successfully?
-        if (hret.first)
-            return hret;
-
-        throw ConfigurationException("Configured Shibboleth handler failed to process the request.");
+        try {
+            pair<bool, long> hret = handler->run(request);
+            // Did the handler run successfully?
+            if (hret.first)
+                return hret;
+            throw ConfigurationException("Configured Shibboleth handler failed to process the request.");
+        }
+        catch (XMLToolingException& ex) {
+            if (!ex.getProperty("eventType") && handler->getEventType())
+                ex.addProperty("eventType", handler->getEventType());
+            throw;
+        }
     }
-    catch (exception& e) {
+    catch (const exception& e) {
         request.log(SPRequest::SPError, e.what());
         Session* session = nullptr;
         try {
             session = request.getSession(false, true, false);   // do not cache
         }
-        catch (exception&) {
+        catch (const exception&) {
         }
         Locker slocker(session, false); // pop existing lock on exit
         TemplateParameters tp(&e, nullptr, session);
