@@ -188,20 +188,6 @@ namespace shibsp {
         Lockable* lock() {return this;}
         void unlock() {}
 
-        // deprecated method
-        ResolutionContext* createResolutionContext(
-            const Application& application,
-            const EntityDescriptor* issuer,
-            const XMLCh* protocol,
-            const NameID* nameid=nullptr,
-            const XMLCh* authncontext_class=nullptr,
-            const XMLCh* authncontext_decl=nullptr,
-            const vector<const opensaml::Assertion*>* tokens=nullptr,
-            const vector<shibsp::Attribute*>* attributes=nullptr
-            ) const {
-            return createResolutionContext(application, nullptr, issuer, protocol, nameid, authncontext_class, authncontext_decl, tokens);
-        }
-
         ResolutionContext* createResolutionContext(
             const Application& application,
             const GenericRequest* request,
@@ -223,7 +209,8 @@ namespace shibsp {
         void resolveAttributes(ResolutionContext& ctx) const;
 
         void getAttributeIds(vector<string>& attributes) const {
-            // Nothing to do, only the extractor would actually generate them.
+            if (!m_exceptionId.empty())
+                attributes.push_back(m_exceptionId.front());
         }
 
     private:
@@ -238,7 +225,7 @@ namespace shibsp {
         vector<string> m_exceptionId;
     };
 
-    AttributeResolver* SHIBSP_DLLLOCAL QueryResolverFactory(const DOMElement* const & e)
+    AttributeResolver* SHIBSP_DLLLOCAL QueryResolverFactory(const DOMElement* const & e, bool)
     {
         return new QueryResolver(e);
     }
@@ -277,7 +264,7 @@ QueryResolver::QueryResolver(const DOMElement* e)
                 }
             }
         }
-        catch (exception& ex) {
+        catch (const exception& ex) {
             m_log.error("exception loading attribute designator: %s", ex.what());
         }
         child = XMLHelper::getNextSiblingElement(child);
@@ -299,7 +286,7 @@ void QueryResolver::SAML1Query(QueryContext& ctx) const
         find_if(ctx.getEntityDescriptor()->getAttributeAuthorityDescriptors(), isValidForProtocol(ctx.getProtocol()));
     if (!AA) {
         m_log.warn("no SAML 1.%d AttributeAuthority role found in metadata", version);
-        return;
+        throw MetadataException("Unable to locate SAML 1 AttributeAuthority role.");
     }
 
     const Application& application = ctx.getApplication();
@@ -347,7 +334,7 @@ void QueryResolver::SAML1Query(QueryContext& ctx) const
             client.sendSAML(request, application.getId(), mcc, loc.get());
             response.reset(client.receiveSAML());
         }
-        catch (exception& ex) {
+        catch (const exception& ex) {
             m_log.error("exception during SAML query to %s: %s", loc.get(), ex.what());
             soaper.reset();
         }
@@ -393,7 +380,7 @@ void QueryResolver::SAML1Query(QueryContext& ctx) const
         if (!policy->isAuthenticated())
             throw SecurityPolicyException("Security of SAML 1.x query result not established.");
     }
-    catch (exception& ex) {
+    catch (const exception& ex) {
         m_log.error("assertion failed policy validation: %s", ex.what());
         throw;
     }
@@ -435,7 +422,7 @@ void QueryResolver::SAML1Query(QueryContext& ctx) const
             filter->filterAttributes(fc, ctx.getResolvedAttributes());
         }
     }
-    catch (exception& ex) {
+    catch (const exception& ex) {
         m_log.error("caught exception extracting/filtering attributes from query result: %s", ex.what());
         for_each(ctx.getResolvedAttributes().begin(), ctx.getResolvedAttributes().end(), xmltooling::cleanup<shibsp::Attribute>());
         ctx.getResolvedAttributes().clear();
@@ -453,7 +440,7 @@ void QueryResolver::SAML2Query(QueryContext& ctx) const
         find_if(ctx.getEntityDescriptor()->getAttributeAuthorityDescriptors(), isValidForProtocol(samlconstants::SAML20P_NS));
     if (!AA) {
         m_log.warn("no SAML 2 AttributeAuthority role found in metadata");
-        return;
+        throw MetadataException("Unable to locate SAML 2.0 AttributeAuthority role.");
     }
 
     const Application& application = ctx.getApplication();
@@ -484,17 +471,30 @@ void QueryResolver::SAML2Query(QueryContext& ctx) const
             auto_ptr<saml2::Subject> subject(saml2::SubjectBuilder::buildSubject());
 
             // Encrypt the NameID?
-            if (encryption.first && (!strcmp(encryption.second, "true") || !strcmp(encryption.second, "back"))) {
-                auto_ptr<EncryptedID> encrypted(EncryptedIDBuilder::buildEncryptedID());
-                encrypted->encrypt(
-                    *ctx.getNameID(),
-                    *(application.getMetadataProvider()),
-                    mcc,
-                    false,
-                    relyingParty->getXMLString("encryptionAlg").second
-                    );
-                subject->setEncryptedID(encrypted.get());
-                encrypted.release();
+            if (SPConfig::shouldSignOrEncrypt(encryption.first ? encryption.second : "conditional", loc.get(), false)) {
+                try {
+                    auto_ptr<EncryptedID> encrypted(EncryptedIDBuilder::buildEncryptedID());
+                    encrypted->encrypt(
+                        *ctx.getNameID(),
+                        *(application.getMetadataProvider()),
+                        mcc,
+                        false,
+                        relyingParty->getXMLString("encryptionAlg").second
+                        );
+                    subject->setEncryptedID(encrypted.get());
+                    encrypted.release();
+                }
+                catch (const std::exception& ex) {
+                    // If we're encrypting deliberately, failure should be fatal.
+                    if (encryption.first && strcmp(encryption.second, "conditional")) {
+                        throw;
+                    }
+                    // If opportunistically, just log and move on.
+                    m_log.info("Conditional encryption of NameID in AttributeQuery failed: %s", ex.what());
+                    auto_ptr<NameID> namewrapper(ctx.getNameID()->cloneNameID());
+                    subject->setNameID(namewrapper.get());
+                    namewrapper.release();
+                }
             }
             else {
                 auto_ptr<NameID> namewrapper(ctx.getNameID()->cloneNameID());
@@ -517,7 +517,7 @@ void QueryResolver::SAML2Query(QueryContext& ctx) const
             client.sendSAML(query, application.getId(), mcc, loc.get());
             srt.reset(client.receiveSAML());
         }
-        catch (exception& ex) {
+        catch (const exception& ex) {
             m_log.error("exception during SAML query to %s: %s", loc.get(), ex.what());
             soaper.reset();
         }
@@ -581,7 +581,7 @@ void QueryResolver::SAML2Query(QueryContext& ctx) const
                     m_log.debugStream() << "decrypted assertion: " << *newtoken << logging::eol;
             }
         }
-        catch (exception& ex) {
+        catch (const exception& ex) {
             m_log.error("failed to decrypt assertion: %s", ex.what());
             throw;
         }
@@ -648,7 +648,7 @@ void QueryResolver::SAML2Query(QueryContext& ctx) const
             }
         }
     }
-    catch (exception& ex) {
+    catch (const exception& ex) {
         m_log.error("assertion failed policy validation: %s", ex.what());
         throw;
     }
@@ -677,7 +677,7 @@ void QueryResolver::SAML2Query(QueryContext& ctx) const
             filter->filterAttributes(fc, ctx.getResolvedAttributes());
         }
     }
-    catch (exception& ex) {
+    catch (const exception& ex) {
         m_log.error("caught exception extracting/filtering attributes from query result: %s", ex.what());
         for_each(ctx.getResolvedAttributes().begin(), ctx.getResolvedAttributes().end(), xmltooling::cleanup<shibsp::Attribute>());
         ctx.getResolvedAttributes().clear();
@@ -716,7 +716,7 @@ void QueryResolver::resolveAttributes(ResolutionContext& ctx) const
             m_log.warn("can't attempt attribute query, either no NameID or no metadata to use");
         }
     }
-    catch (exception& ex) {
+    catch (const exception& ex) {
         // Already logged.
         if (!m_exceptionId.empty()) {
             auto_ptr<SimpleAttribute> attr(new SimpleAttribute(m_exceptionId));
@@ -724,5 +724,8 @@ void QueryResolver::resolveAttributes(ResolutionContext& ctx) const
             qctx.getResolvedAttributes().push_back(attr.get());
             attr.release();
         }
+        else {
+            throw; // not exposing the exception as an attribute, so just surface to caller
+        }
     }
 }
