@@ -160,26 +160,12 @@ namespace shibsp {
     class SHIBSP_DLLLOCAL SimpleAggregationResolver : public AttributeResolver
     {
     public:
-        SimpleAggregationResolver(const DOMElement* e);
+        SimpleAggregationResolver(const DOMElement* e, bool deprecationSupport=true);
         ~SimpleAggregationResolver() {}
 
         Lockable* lock() {return this;}
         void unlock() {}
 
-        // deprecated method
-        ResolutionContext* createResolutionContext(
-            const Application& application,
-            const EntityDescriptor* issuer,
-            const XMLCh* protocol,
-            const NameID* nameid=nullptr,
-            const XMLCh* authncontext_class=nullptr,
-            const XMLCh* authncontext_decl=nullptr,
-            const vector<const opensaml::Assertion*>* tokens=nullptr,
-            const vector<shibsp::Attribute*>* attributes=nullptr
-            ) const {
-            return createResolutionContext(application, nullptr, issuer, protocol, nameid, authncontext_class, authncontext_decl, tokens, attributes);
-        }
-
         ResolutionContext* createResolutionContext(
             const Application& application,
             const GenericRequest* request,
@@ -205,6 +191,8 @@ namespace shibsp {
         void getAttributeIds(vector<string>& attributes) const {
             if (m_extractor)
                 m_extractor->getAttributeIds(attributes);
+            if (!m_exceptionId.empty())
+                attributes.push_back(m_exceptionId.front());
         }
 
     private:
@@ -224,9 +212,9 @@ namespace shibsp {
         vector<string> m_exceptionId;
     };
 
-    AttributeResolver* SHIBSP_DLLLOCAL SimpleAggregationResolverFactory(const DOMElement* const & e)
+    AttributeResolver* SHIBSP_DLLLOCAL SimpleAggregationResolverFactory(const DOMElement* const & e, bool deprecationSupport)
     {
-        return new SimpleAggregationResolver(e);
+        return new SimpleAggregationResolver(e, deprecationSupport);
     }
 
     static const XMLCh _AttributeExtractor[] =  UNICODE_LITERAL_18(A,t,t,r,i,b,u,t,e,E,x,t,r,a,c,t,o,r);
@@ -243,8 +231,8 @@ namespace shibsp {
     static const XMLCh _type[] =                UNICODE_LITERAL_4(t,y,p,e);
 };
 
-SimpleAggregationResolver::SimpleAggregationResolver(const DOMElement* e)
-    : m_log(Category::getInstance(SHIBSP_LOGCAT".AttributeResolver.SimpleAggregation")),
+SimpleAggregationResolver::SimpleAggregationResolver(const DOMElement* e, bool deprecationSupport)
+    : m_log(Category::getInstance(SHIBSP_LOGCAT ".AttributeResolver.SimpleAggregation")),
         m_policyId(XMLHelper::getAttrString(e, nullptr, policyId)),
         m_subjectMatch(XMLHelper::getAttrBool(e, false, subjectMatch))
 {
@@ -256,6 +244,7 @@ SimpleAggregationResolver::SimpleAggregationResolver(const DOMElement* e)
     if (aid && *aid) {
         auto_ptr_char dup(aid);
         string sdup(dup.get());
+        trim(sdup);
         split(m_attributeIds, sdup, is_space(), algorithm::token_compress_on);
 
         aid = e->getAttributeNS(nullptr, format);
@@ -273,7 +262,7 @@ SimpleAggregationResolver::SimpleAggregationResolver(const DOMElement* e)
         if (t.empty())
             throw ConfigurationException("MetadataProvider element missing type attribute.");
         m_log.info("building MetadataProvider of type %s...", t.c_str());
-        m_metadata.reset(SAMLConfig::getConfig().MetadataProviderManager.newPlugin(t.c_str(), child));
+        m_metadata.reset(SAMLConfig::getConfig().MetadataProviderManager.newPlugin(t.c_str(), child, deprecationSupport));
         m_metadata->init();
     }
 
@@ -283,7 +272,7 @@ SimpleAggregationResolver::SimpleAggregationResolver(const DOMElement* e)
         if (t.empty())
             throw ConfigurationException("TrustEngine element missing type attribute.");
         m_log.info("building TrustEngine of type %s...", t.c_str());
-        m_trust.reset(XMLToolingConfig::getConfig().TrustEngineManager.newPlugin(t.c_str(), child));
+        m_trust.reset(XMLToolingConfig::getConfig().TrustEngineManager.newPlugin(t.c_str(), child, deprecationSupport));
     }
 
     child = XMLHelper::getFirstChildElement(e,  _AttributeExtractor);
@@ -292,7 +281,7 @@ SimpleAggregationResolver::SimpleAggregationResolver(const DOMElement* e)
         if (t.empty())
             throw ConfigurationException("AttributeExtractor element missing type attribute.");
         m_log.info("building AttributeExtractor of type %s...", t.c_str());
-        m_extractor.reset(SPConfig::getConfig().AttributeExtractorManager.newPlugin(t.c_str(), child));
+        m_extractor.reset(SPConfig::getConfig().AttributeExtractorManager.newPlugin(t.c_str(), child, deprecationSupport));
     }
 
     child = XMLHelper::getFirstChildElement(e,  _AttributeFilter);
@@ -301,7 +290,7 @@ SimpleAggregationResolver::SimpleAggregationResolver(const DOMElement* e)
         if (t.empty())
             throw ConfigurationException("AttributeFilter element missing type attribute.");
         m_log.info("building AttributeFilter of type %s...", t.c_str());
-        m_filter.reset(SPConfig::getConfig().AttributeFilterManager.newPlugin(t.c_str(), child));
+        m_filter.reset(SPConfig::getConfig().AttributeFilterManager.newPlugin(t.c_str(), child, deprecationSupport));
     }
 
     child = XMLHelper::getFirstChildElement(e);
@@ -329,7 +318,7 @@ SimpleAggregationResolver::SimpleAggregationResolver(const DOMElement* e)
                     obj.release();
                 }
             }
-            catch (std::exception& ex) {
+            catch (const std::exception& ex) {
                 m_log.error("exception loading attribute designator: %s", ex.what());
             }
         }
@@ -350,11 +339,11 @@ void SimpleAggregationResolver::doQuery(SimpleAggregationContext& ctx, const cha
         (m_metadata ? m_metadata.get() : application.getMetadataProvider())->getEntityDescriptor(mc);
     if (!mdresult.first) {
         m_log.warn("unable to locate metadata for provider (%s)", entityID);
-        return;
+        throw MetadataException("Unable to locate metadata for provider ($entityID)", namedparams(1, "entityID", entityID));
     }
     else if (!(AA=dynamic_cast<const AttributeAuthorityDescriptor*>(mdresult.second))) {
         m_log.warn("no SAML 2 AttributeAuthority role found in metadata for (%s)", entityID);
-        return;
+        throw MetadataException("Unable to locate SAML 2.0 AttributeAuthority role for provider ($entityID)", namedparams(1, "entityID", entityID));
     }
 
     const PropertySet* relyingParty = application.getRelyingParty(mdresult.first);
@@ -389,20 +378,35 @@ void SimpleAggregationResolver::doQuery(SimpleAggregationContext& ctx, const cha
             auto_ptr<saml2::Subject> subject(saml2::SubjectBuilder::buildSubject());
 
             // Encrypt the NameID?
-            if (encryption.first && (!strcmp(encryption.second, "true") || !strcmp(encryption.second, "back"))) {
-                auto_ptr<EncryptedID> encrypted(EncryptedIDBuilder::buildEncryptedID());
-                encrypted->encrypt(
-                    *name,
-                    *(policy->getMetadataProvider()),
-                    mcc,
-                    false,
-                    relyingParty->getXMLString("encryptionAlg").second
+            if (SPConfig::shouldSignOrEncrypt(encryption.first ? encryption.second : "conditional", loc.get(), false)) {
+                try {
+                    auto_ptr<EncryptedID> encrypted(EncryptedIDBuilder::buildEncryptedID());
+                    encrypted->encrypt(
+                        *name,
+                        *(policy->getMetadataProvider()),
+                        mcc,
+                        false,
+                        relyingParty->getXMLString("encryptionAlg").second
                     );
-                subject->setEncryptedID(encrypted.get());
-                encrypted.release();
+                    subject->setEncryptedID(encrypted.get());
+                    encrypted.release();
+                }
+                catch (const std::exception& ex) {
+                    // If we're encrypting deliberately, failure should be fatal.
+                    if (encryption.first && strcmp(encryption.second, "conditional")) {
+                        throw;
+                    }
+                    // If opportunistically, just log and move on.
+                    m_log.info("Conditional encryption of NameID in AttributeQuery failed: %s", ex.what());
+                    auto_ptr<NameID> namewrapper(name->cloneNameID());
+                    subject->setNameID(namewrapper.get());
+                    namewrapper.release();
+                }
             }
             else {
-                subject->setNameID(name->cloneNameID());
+                auto_ptr<NameID> namewrapper(name->cloneNameID());
+                subject->setNameID(namewrapper.get());
+                namewrapper.release();
             }
 
             saml2p::AttributeQuery* query = saml2p::AttributeQueryBuilder::buildAttributeQuery();
@@ -420,7 +424,7 @@ void SimpleAggregationResolver::doQuery(SimpleAggregationContext& ctx, const cha
             client.sendSAML(query, application.getId(), mcc, loc.get());
             srt.reset(client.receiveSAML());
         }
-        catch (std::exception& ex) {
+        catch (const std::exception& ex) {
             m_log.error("exception during SAML query to %s: %s", loc.get(), ex.what());
             soaper.reset();
         }
@@ -485,7 +489,7 @@ void SimpleAggregationResolver::doQuery(SimpleAggregationContext& ctx, const cha
                     m_log.debugStream() << "decrypted assertion: " << *newtoken << logging::eol;
             }
         }
-        catch (std::exception& ex) {
+        catch (const std::exception& ex) {
             m_log.error("failed to decrypt assertion: %s", ex.what());
             throw;
         }
@@ -552,7 +556,7 @@ void SimpleAggregationResolver::doQuery(SimpleAggregationContext& ctx, const cha
             }
         }
     }
-    catch (std::exception& ex) {
+    catch (const std::exception& ex) {
         m_log.error("assertion failed policy validation: %s", ex.what());
         throw;
     }
@@ -581,7 +585,7 @@ void SimpleAggregationResolver::doQuery(SimpleAggregationContext& ctx, const cha
             filter->filterAttributes(fc, ctx.getResolvedAttributes());
         }
     }
-    catch (std::exception& ex) {
+    catch (const std::exception& ex) {
         m_log.error("caught exception extracting/filtering attributes from query result: %s", ex.what());
         for_each(ctx.getResolvedAttributes().begin(), ctx.getResolvedAttributes().end(), xmltooling::cleanup<shibsp::Attribute>());
         ctx.getResolvedAttributes().clear();
@@ -687,7 +691,7 @@ void SimpleAggregationResolver::resolveAttributes(ResolutionContext& ctx) const
                 try {
                     doQuery(qctx, source->first.c_str(), n ? n.get() : qctx.getNameID());
                 }
-                catch (std::exception& ex) {
+                catch (const std::exception& ex) {
                     if (exceptAttr.get())
                         exceptAttr->getValues().push_back(XMLToolingConfig::getConfig().getURLEncoder()->encode(ex.what()));
                 }
@@ -711,7 +715,7 @@ void SimpleAggregationResolver::resolveAttributes(ResolutionContext& ctx) const
                             try {
                                 doQuery(qctx, link->c_str(), n ? n.get() : qctx.getNameID());
                             }
-                            catch (std::exception& ex) {
+                            catch (const std::exception& ex) {
                                 if (exceptAttr.get())
                                     exceptAttr->getValues().push_back(XMLToolingConfig::getConfig().getURLEncoder()->encode(ex.what()));
                             }
@@ -736,7 +740,7 @@ void SimpleAggregationResolver::resolveAttributes(ResolutionContext& ctx) const
                                 try {
                                     doQuery(qctx, link->c_str(), n ? n.get() : qctx.getNameID());
                                 }
-                                catch (std::exception& ex) {
+                                catch (const std::exception& ex) {
                                     if (exceptAttr.get())
                                         exceptAttr->getValues().push_back(XMLToolingConfig::getConfig().getURLEncoder()->encode(ex.what()));
                                 }
