@@ -166,6 +166,11 @@ const XMLCh* Handler::getProtocolFamily() const
     return nullptr;
 }
 
+const char* Handler::getEventType() const
+{
+    return nullptr;
+}
+
 void Handler::log(SPRequest::SPLogLevel level, const string& msg) const
 {
     Category::getInstance(SHIBSP_LOGCAT ".Handler").log(
@@ -413,8 +418,8 @@ void Handler::recoverRelayState(
 }
 
 AbstractHandler::AbstractHandler(
-    const DOMElement* e, Category& log, DOMNodeFilter* filter, const map<string,string>* remapper
-    ) : m_log(log), m_configNS(shibspconstants::SHIB2SPCONFIG_NS) {
+    const DOMElement* e, Category& log, DOMNodeFilter* filter, const Remapper* remapper
+    ) : m_log(log) {
     load(e, nullptr, filter, remapper);
 }
 
@@ -481,7 +486,7 @@ void AbstractHandler::fillStatus(saml2p::StatusResponseType& response, const XML
         ssubcode->setValue(subcode);
     }
     if (msg) {
-        pair<bool,bool> flag = getBool("detailedErrors", m_configNS.get());
+        pair<bool,bool> flag = getBool("detailedErrors", shibspconstants::ASCII_SHIBSPCONFIG_NS);
         auto_ptr_XMLCh widemsg((flag.first && flag.second) ? msg : "Error processing request.");
         saml2p::StatusMessage* sm = saml2p::StatusMessageBuilder::buildStatusMessage();
         status->setStatusMessage(sm);
@@ -498,15 +503,17 @@ long AbstractHandler::sendMessage(
     const saml2md::RoleDescriptor* role,
     const Application& application,
     HTTPResponse& httpResponse,
-    bool signIfPossible
+    const char* defaultSigningProperty
     ) const
 {
     const EntityDescriptor* entity = role ? dynamic_cast<const EntityDescriptor*>(role->getParent()) : nullptr;
     const PropertySet* relyingParty = application.getRelyingParty(entity);
-    pair<bool,const char*> flag = signIfPossible ? make_pair(true,(const char*)"true") : relyingParty->getString("signing");
-    if (flag.first && (!strcmp(flag.second, "true") ||
-                        (encoder.isUserAgentPresent() && !strcmp(flag.second, "front")) ||
-                        (!encoder.isUserAgentPresent() && !strcmp(flag.second, "back")))) {
+    pair<bool, const char*> flag = getString("signing");
+    if (!flag.first)
+        flag = getString("signing", shibspconstants::ASCII_SHIBSPCONFIG_NS);
+    if (!flag.first)
+        flag = relyingParty->getString("signing");
+    if (SPConfig::shouldSignOrEncrypt(flag.first ? flag.second : defaultSigningProperty, destination, encoder.isUserAgentPresent())) {
         CredentialResolver* credResolver = application.getCredentialResolver();
         if (credResolver) {
             Locker credLocker(credResolver);
@@ -639,8 +646,35 @@ void AbstractHandler::preservePostData(
             postkey = string(mech.second-3) + ':' + out.string();
         }
 
-        // Set a cookie with key info.
         pair<string,const char*> shib_cookie = getPostCookieNameProps(application, relayState);
+
+        // Purge any cookies in excess of 25.
+        int maxCookies = 25,purgedCookies = 0;
+        string exp;
+
+        // Walk the list of cookies backwards by name.
+        const map<string,string>& cookies = request.getCookies();
+        for (map<string,string>::const_reverse_iterator i = cookies.rbegin(); i != cookies.rend(); ++i) {
+            // Process post data cookies only.
+            if (starts_with(i->first, "_shibpost_")) {
+                if (maxCookies > 0) {
+                    // Keep it, but count it against the limit.
+                    --maxCookies;
+                }
+                else {
+                    // We're over the limit, so everything here and older gets cleaned up.
+                    if (exp.empty())
+                        exp = string(shib_cookie.second) + "; expires=Mon, 01 Jan 2001 00:00:00 GMT";
+                    response.setCookie(i->first.c_str(), exp.c_str());
+                    ++purgedCookies;
+                }
+            }
+        }
+
+        if (purgedCookies > 0)
+            log(SPRequest::SPDebug, string("purged ") + lexical_cast<string>(purgedCookies) + " stale POST preservation cookie(s) from client");
+
+        // Set a cookie with key info.
         postkey += shib_cookie.second;
         response.setCookie(shib_cookie.first.c_str(), postkey.c_str());
     }
@@ -802,7 +836,7 @@ DDF AbstractHandler::getPostData(const Application& application, const HTTPReque
     return DDF();
 }
 
-pair<bool,bool> AbstractHandler::getBool(const char* name, const SPRequest& request, unsigned int type) const
+pair<bool,bool> AbstractHandler::getBool(const char* name, const HTTPRequest& request, unsigned int type) const
 {
     if (type & HANDLER_PROPERTY_REQUEST) {
         const char* param = request.getParameter(name);
@@ -810,8 +844,9 @@ pair<bool,bool> AbstractHandler::getBool(const char* name, const SPRequest& requ
             return make_pair(true, (*param=='t' || *param=='1'));
     }
     
-    if (type & HANDLER_PROPERTY_MAP) {
-        pair<bool,bool> ret = request.getRequestSettings().first->getBool(name);
+    const SPRequest* sprequest = dynamic_cast<const SPRequest*>(&request);
+    if (sprequest && (type & HANDLER_PROPERTY_MAP)) {
+        pair<bool,bool> ret = sprequest->getRequestSettings().first->getBool(name);
         if (ret.first)
             return ret;
     }
@@ -823,7 +858,7 @@ pair<bool,bool> AbstractHandler::getBool(const char* name, const SPRequest& requ
     return make_pair(false,false);
 }
 
-pair<bool,const char*> AbstractHandler::getString(const char* name, const SPRequest& request, unsigned int type) const
+pair<bool,const char*> AbstractHandler::getString(const char* name, const HTTPRequest& request, unsigned int type) const
 {
     if (type & HANDLER_PROPERTY_REQUEST) {
         const char* param = request.getParameter(name);
@@ -831,8 +866,9 @@ pair<bool,const char*> AbstractHandler::getString(const char* name, const SPRequ
             return make_pair(true, param);
     }
     
-    if (type & HANDLER_PROPERTY_MAP) {
-        pair<bool,const char*> ret = request.getRequestSettings().first->getString(name);
+    const SPRequest* sprequest = dynamic_cast<const SPRequest*>(&request);
+    if (sprequest && (type & HANDLER_PROPERTY_MAP)) {
+        pair<bool,const char*> ret = sprequest->getRequestSettings().first->getString(name);
         if (ret.first)
             return ret;
     }
@@ -844,7 +880,7 @@ pair<bool,const char*> AbstractHandler::getString(const char* name, const SPRequ
     return pair<bool,const char*>(false,nullptr);
 }
 
-pair<bool,unsigned int> AbstractHandler::getUnsignedInt(const char* name, const SPRequest& request, unsigned int type) const
+pair<bool,unsigned int> AbstractHandler::getUnsignedInt(const char* name, const HTTPRequest& request, unsigned int type) const
 {
     if (type & HANDLER_PROPERTY_REQUEST) {
         const char* param = request.getParameter(name);
@@ -858,8 +894,9 @@ pair<bool,unsigned int> AbstractHandler::getUnsignedInt(const char* name, const
         }
     }
     
-    if (type & HANDLER_PROPERTY_MAP) {
-        pair<bool,unsigned int> ret = request.getRequestSettings().first->getUnsignedInt(name);
+    const SPRequest* sprequest = dynamic_cast<const SPRequest*>(&request);
+    if (sprequest && (type & HANDLER_PROPERTY_MAP)) {
+        pair<bool,unsigned int> ret = sprequest->getRequestSettings().first->getUnsignedInt(name);
         if (ret.first)
             return ret;
     }
@@ -871,7 +908,7 @@ pair<bool,unsigned int> AbstractHandler::getUnsignedInt(const char* name, const
     return pair<bool,unsigned int>(false,0);
 }
 
-pair<bool,int> AbstractHandler::getInt(const char* name, const SPRequest& request, unsigned int type) const
+pair<bool,int> AbstractHandler::getInt(const char* name, const HTTPRequest& request, unsigned int type) const
 {
     if (type & HANDLER_PROPERTY_REQUEST) {
         const char* param = request.getParameter(name);
@@ -879,8 +916,9 @@ pair<bool,int> AbstractHandler::getInt(const char* name, const SPRequest& reques
             return pair<bool,int>(true, atoi(param));
     }
     
-    if (type & HANDLER_PROPERTY_MAP) {
-        pair<bool,int> ret = request.getRequestSettings().first->getInt(name);
+    const SPRequest* sprequest = dynamic_cast<const SPRequest*>(&request);
+    if (sprequest && (type & HANDLER_PROPERTY_MAP)) {
+        pair<bool,int> ret = sprequest->getRequestSettings().first->getInt(name);
         if (ret.first)
             return ret;
     }
