@@ -29,6 +29,7 @@
 #include "Application.h"
 #include "ServiceProvider.h"
 #include "SPRequest.h"
+#include "TransactionLog.h"
 #include "handler/AssertionConsumerService.h"
 #include "util/CGIParser.h"
 #include "util/SPConstants.h"
@@ -73,7 +74,7 @@ using namespace boost;
 using namespace std;
 
 AssertionConsumerService::AssertionConsumerService(
-    const DOMElement* e, const char* appId, Category& log, DOMNodeFilter* filter, const map<string,string>* remapper
+    const DOMElement* e, const char* appId, Category& log, DOMNodeFilter* filter, const Remapper* remapper, bool deprecationSupport
     ) : AbstractHandler(e, log, filter, remapper)
 {
     if (!e)
@@ -83,11 +84,7 @@ AssertionConsumerService::AssertionConsumerService(
     setAddress(address.c_str());
 #ifndef SHIBSP_LITE
     if (SPConfig::getConfig().isEnabled(SPConfig::OutOfProcess)) {
-        m_decoder.reset(
-            SAMLConfig::getConfig().MessageDecoderManager.newPlugin(
-                getString("Binding").second, pair<const DOMElement*,const XMLCh*>(e,shibspconstants::SHIB2SPCONFIG_NS)
-                )
-            );
+        m_decoder.reset(SAMLConfig::getConfig().MessageDecoderManager.newPlugin(getString("Binding").second, e, deprecationSupport));
         m_decoder->setArtifactResolver(SPConfig::getConfig().getArtifactResolver());
     }
 #endif
@@ -124,7 +121,7 @@ pair<bool,long> AssertionConsumerService::run(SPRequest& request, bool isHandler
         headers.push_back("Accept-Language");
         DDF out,in = wrap(request, &headers);
         DDFJanitor jin(in), jout(out);
-        out = request.getServiceProvider().getListenerService()->send(in);
+        out = send(request, in);
         return unwrap(request, out);
     }
 }
@@ -161,7 +158,7 @@ pair<bool,long> AssertionConsumerService::processMessage(
 {
 #ifndef SHIBSP_LITE
     // Locate policy key.
-    pair<bool,const char*> prop = getString("policyId", m_configNS.get());  // may be namespace-qualified if inside handler element
+    pair<bool,const char*> prop = getString("policyId", shibspconstants::ASCII_SHIBSPCONFIG_NS);  // may be namespace-qualified if inside handler element
     if (!prop.first)
         prop = getString("policyId");   // try unqualified
     if (!prop.first)
@@ -217,7 +214,7 @@ pair<bool,long> AssertionConsumerService::processMessage(
                     recoverRelayState(application, httpRequest, httpResponse, relayState, false);
                     hook += "&target=" + encoder->encode(relayState.c_str());
                 }
-                catch (std::exception& ex) {
+                catch (const std::exception& ex) {
                     m_log.warn("error recovering relay state: %s", ex.what());
                 }
             }
@@ -228,12 +225,14 @@ pair<bool,long> AssertionConsumerService::processMessage(
         return finalizeResponse(application, httpRequest, httpResponse, relayState);
     }
     catch (XMLToolingException& ex) {
+        m_log.warn("error processing incoming assertion: %s", ex.what());
+
         // Recover relay state.
         if (!relayState.empty()) {
             try {
                 recoverRelayState(application, httpRequest, httpResponse, relayState, false);
             }
-            catch (std::exception& rsex) {
+            catch (const std::exception& rsex) {
                 m_log.warn("error recovering relay state: %s", rsex.what());
                 relayState.erase();
                 recoverRelayState(application, httpRequest, httpResponse, relayState, false);
@@ -243,7 +242,7 @@ pair<bool,long> AssertionConsumerService::processMessage(
         // Check for isPassive error condition.
         const char* sc2 = ex.getProperty("statusCode2");
         if (sc2 && !strcmp(sc2, "urn:oasis:names:tc:SAML:2.0:status:NoPassive")) {
-            pair<bool,bool> ignore = getBool("ignoreNoPassive", m_configNS.get());  // may be namespace-qualified inside handler element
+            pair<bool,bool> ignore = getBool("ignoreNoPassive", shibspconstants::ASCII_SHIBSPCONFIG_NS);  // may be namespace-qualified inside handler element
             if (!ignore.first)
                 ignore = getBool("ignoreNoPassive");    // try unqualified
             if (ignore.first && ignore.second && !relayState.empty()) {
@@ -258,7 +257,7 @@ pair<bool,long> AssertionConsumerService::processMessage(
 
         // Log the error.
         try {
-            scoped_ptr<TransactionLog::Event> event(SPConfig::getConfig().EventManager.newPlugin(LOGIN_EVENT, nullptr));
+            scoped_ptr<TransactionLog::Event> event(SPConfig::getConfig().EventManager.newPlugin(LOGIN_EVENT, nullptr, false));
             LoginEvent* error_event = dynamic_cast<LoginEvent*>(event.get());
             if (error_event) {
                 error_event->m_exception = &ex;
@@ -278,7 +277,7 @@ pair<bool,long> AssertionConsumerService::processMessage(
                 m_log.warn("unable to audit event, log event object was of an incorrect type");
             }
         }
-        catch (std::exception& ex2) {
+        catch (const std::exception& ex2) {
             m_log.warn("exception auditing event: %s", ex2.what());
         }
 
@@ -338,6 +337,11 @@ void AssertionConsumerService::checkAddress(const Application& application, cons
     }
 }
 
+const char* AssertionConsumerService::getEventType() const
+{
+    return LOGIN_EVENT;
+}
+
 #ifndef SHIBSP_LITE
 
 const XMLCh* AssertionConsumerService::getProtocolFamily() const
@@ -355,7 +359,7 @@ void AssertionConsumerService::generateMetadata(SPSSODescriptor& role, const cha
     // Initial guess at index to use.
     pair<bool,unsigned int> ix = pair<bool,unsigned int>(false,0);
     if (!strncmp(handlerURL, "https", 5))
-        ix = getUnsignedInt("sslIndex", shibspconstants::ASCII_SHIB2SPCONFIG_NS);
+        ix = getUnsignedInt("sslIndex", shibspconstants::ASCII_SHIBSPCONFIG_NS);
     if (!ix.first)
         ix = getUnsignedInt("index");
     if (!ix.first)
@@ -380,13 +384,6 @@ void AssertionConsumerService::generateMetadata(SPSSODescriptor& role, const cha
     role.getAssertionConsumerServices().push_back(ep);
 }
 
-opensaml::SecurityPolicy* AssertionConsumerService::createSecurityPolicy(
-    const Application& application, const xmltooling::QName* role, bool validate, const char* policyId
-    ) const
-{
-    return new SecurityPolicy(application, role, validate, policyId);
-}
-
 namespace {
     class SHIBSP_DLLLOCAL DummyContext : public ResolutionContext
     {
@@ -413,33 +410,6 @@ namespace {
 
 vector<Assertion*> DummyContext::m_tokens;
 
-ResolutionContext* AssertionConsumerService::resolveAttributes(
-    const Application& application,
-    const saml2md::RoleDescriptor* issuer,
-    const XMLCh* protocol,
-    const saml1::NameIdentifier* v1nameid,
-    const saml2::NameID* nameid,
-    const XMLCh* authncontext_class,
-    const XMLCh* authncontext_decl,
-    const vector<const Assertion*>* tokens
-    ) const
-{
-    return resolveAttributes(
-        application,
-        nullptr,
-        issuer,
-        protocol,
-        nullptr,
-        v1nameid,
-        nullptr,
-        nameid,
-        nullptr,
-        authncontext_class,
-        authncontext_decl,
-        tokens
-        );
-}
-
 ResolutionContext* AssertionConsumerService::resolveAttributes(
     const Application& application,
     const GenericRequest* request,
@@ -474,7 +444,7 @@ ResolutionContext* AssertionConsumerService::resolveAttributes(
                             *id = mprefix.second + *id;
                     }
                 }
-                catch (std::exception& ex) {
+                catch (const std::exception& ex) {
                     m_log.error("caught exception extracting attributes: %s", ex.what());
                 }
             }
@@ -486,7 +456,7 @@ ResolutionContext* AssertionConsumerService::resolveAttributes(
             try {
                 extractor->extractAttributes(application, request, issuer, *protmsg, resolvedAttributes);
             }
-            catch (std::exception& ex) {
+            catch (const std::exception& ex) {
                 m_log.error("caught exception extracting attributes: %s", ex.what());
             }
         }
@@ -498,7 +468,7 @@ ResolutionContext* AssertionConsumerService::resolveAttributes(
                 else
                     extractor->extractAttributes(application, request, issuer, *nameid, resolvedAttributes);
             }
-            catch (std::exception& ex) {
+            catch (const std::exception& ex) {
                 m_log.error("caught exception extracting attributes: %s", ex.what());
             }
         }
@@ -510,7 +480,7 @@ ResolutionContext* AssertionConsumerService::resolveAttributes(
                 else
                     extractor->extractAttributes(application, request, issuer, *statement, resolvedAttributes);
             }
-            catch (std::exception& ex) {
+            catch (const std::exception& ex) {
                 m_log.error("caught exception extracting attributes: %s", ex.what());
             }
         }
@@ -521,7 +491,7 @@ ResolutionContext* AssertionConsumerService::resolveAttributes(
                 try {
                     extractor->extractAttributes(application, request, issuer, *t, resolvedAttributes);
                 }
-                catch (std::exception& ex) {
+                catch (const std::exception& ex) {
                     m_log.error("caught exception extracting attributes: %s", ex.what());
                 }
             }
@@ -534,7 +504,7 @@ ResolutionContext* AssertionConsumerService::resolveAttributes(
             try {
                 filter->filterAttributes(fc, resolvedAttributes);
             }
-            catch (std::exception& ex) {
+            catch (const std::exception& ex) {
                 m_log.error("caught exception filtering attributes: %s", ex.what());
                 m_log.error("dumping extracted attributes due to filtering exception");
                 for_each(resolvedAttributes.begin(), resolvedAttributes.end(), xmltooling::cleanup<shibsp::Attribute>());
@@ -574,16 +544,17 @@ ResolutionContext* AssertionConsumerService::resolveAttributes(
             return ctx.release();
         }
     }
-    catch (std::exception& ex) {
-        m_log.error("attribute resolution failed: %s", ex.what());
+    catch (const std::exception&) {
+        // Logging should be handled by the resolver plugin at whatever level is appropriate.
     }
 
     if (!resolvedAttributes.empty()) {
         try {
             return new DummyContext(resolvedAttributes);
         }
-        catch (bad_alloc&) {
+        catch (...) {
             for_each(resolvedAttributes.begin(), resolvedAttributes.end(), xmltooling::cleanup<shibsp::Attribute>());
+            throw;
         }
     }
     return nullptr;
@@ -639,7 +610,7 @@ LoginEvent* AssertionConsumerService::newLoginEvent(const Application& applicati
     if (!SPConfig::getConfig().isEnabled(SPConfig::Logging))
         return nullptr;
     try {
-        auto_ptr<TransactionLog::Event> event(SPConfig::getConfig().EventManager.newPlugin(LOGIN_EVENT, nullptr));
+        auto_ptr<TransactionLog::Event> event(SPConfig::getConfig().EventManager.newPlugin(LOGIN_EVENT, nullptr, false));
         LoginEvent* login_event = dynamic_cast<LoginEvent*>(event.get());
         if (login_event) {
             login_event->m_request = &request;
@@ -652,7 +623,7 @@ LoginEvent* AssertionConsumerService::newLoginEvent(const Application& applicati
             m_log.warn("unable to audit event, log event object was of an incorrect type");
         }
     }
-    catch (std::exception& ex) {
+    catch (const std::exception& ex) {
         m_log.warn("exception auditing event: %s", ex.what());
     }
     return nullptr;
