@@ -53,11 +53,8 @@ LogoutHandler::~LogoutHandler()
 {
 }
 
-pair<bool,long> LogoutHandler::sendLogoutPage(
-    const Application& application, const HTTPRequest& request, HTTPResponse& response, bool local, const char* status
-    ) const
-{
-    return sendLogoutPage(application, request, response, local ? "local" : "global");
+const char* LogoutHandler::getEventType() const {
+    return LOGOUT_EVENT;
 }
 
 pair<bool,long> LogoutHandler::sendLogoutPage(
@@ -294,7 +291,7 @@ LogoutEvent* LogoutHandler::newLogoutEvent(
     if (!SPConfig::getConfig().isEnabled(SPConfig::Logging))
         return nullptr;
     try {
-        auto_ptr<TransactionLog::Event> event(SPConfig::getConfig().EventManager.newPlugin(LOGOUT_EVENT, nullptr));
+        auto_ptr<TransactionLog::Event> event(SPConfig::getConfig().EventManager.newPlugin(LOGOUT_EVENT, nullptr, false));
         LogoutEvent* logout_event = dynamic_cast<LogoutEvent*>(event.get());
         if (logout_event) {
             logout_event->m_request = request;
