@@ -178,7 +178,7 @@ namespace shibsp {
     #pragma warning( pop )
 #endif
 
-    Handler* SHIBSP_DLLLOCAL MetadataGeneratorFactory(const pair<const DOMElement*,const char*>& p)
+    Handler* SHIBSP_DLLLOCAL MetadataGeneratorFactory(const pair<const DOMElement*,const char*>& p, bool)
     {
         return new MetadataGenerator(p.first, p.second);
     }
@@ -186,7 +186,7 @@ namespace shibsp {
 };
 
 MetadataGenerator::MetadataGenerator(const DOMElement* e, const char* appId)
-    : SecuredHandler(e, Category::getInstance(SHIBSP_LOGCAT".MetadataGenerator"))
+    : SecuredHandler(e, Category::getInstance(SHIBSP_LOGCAT ".Handler.Metadata"))
 #ifndef SHIBSP_LITE
         ,m_http(0), m_https(0), m_encryptionBuilder(nullptr), m_digestBuilder(nullptr)
 #endif
@@ -327,30 +327,35 @@ MetadataGenerator::MetadataGenerator(const DOMElement* e, const char* appId)
     }
 
     if (m_digests.empty()) {
+#ifdef XSEC_OPENSSL_HAVE_SHA2
         registerDigestMethod(DSIGConstants::s_unicodeStrURISHA512);
         registerDigestMethod(DSIGConstants::s_unicodeStrURISHA384);
         registerDigestMethod(DSIGConstants::s_unicodeStrURISHA256);
         registerDigestMethod(DSIGConstants::s_unicodeStrURISHA224);
+#endif
         registerDigestMethod(DSIGConstants::s_unicodeStrURISHA1);
     }
 
     if (m_signings.empty()) {
-#ifdef XSEC_OPENSSL_HAVE_EC
+#ifdef XSEC_OPENSSL_HAVE_SHA2
+# ifdef XSEC_OPENSSL_HAVE_EC
         registerSigningMethod(DSIGConstants::s_unicodeStrURIECDSA_SHA512);
         registerSigningMethod(DSIGConstants::s_unicodeStrURIECDSA_SHA384);
         registerSigningMethod(DSIGConstants::s_unicodeStrURIECDSA_SHA256);
-# ifdef URI_ID_ECDSA_SHA224
+#  ifdef URI_ID_ECDSA_SHA224
         registerSigningMethod(DSIGConstants::s_unicodeStrURIECDSA_SHA224);
+#  endif
 # endif
-#endif
         registerSigningMethod(DSIGConstants::s_unicodeStrURIRSA_SHA512);
         registerSigningMethod(DSIGConstants::s_unicodeStrURIRSA_SHA384);
         registerSigningMethod(DSIGConstants::s_unicodeStrURIRSA_SHA256);
 
-#ifdef URI_ID_DSA_SHA256
+# ifdef URI_ID_DSA_SHA256
         registerSigningMethod(DSIGConstants::s_unicodeStrURIDSA_SHA256);
+# endif
 #endif
 
+
 #ifdef XSEC_OPENSSL_HAVE_EC
         registerSigningMethod(DSIGConstants::s_unicodeStrURIECDSA_SHA1);
 #endif
@@ -381,7 +386,7 @@ pair<bool,long> MetadataGenerator::run(SPRequest& request, bool isHandler) const
                 in.addmember("entity_id").string(request.getParameter("entityID"));
             DDFJanitor jin(in), jout(out);
 
-            out = request.getServiceProvider().getListenerService()->send(in);
+            out = send(request, in);
             return unwrap(request, out);
         }
     }
