@@ -45,7 +45,7 @@
 # include <xsec/framework/XSECProvider.hpp>
 #endif
 
-#ifdef HAVE_GSSAPI_NAMINGEXTS
+#if HAVE_DECL_GSS_GET_NAME_ATTRIBUTE
 # ifdef SHIBSP_HAVE_GSSMIT
 #  include <gssapi/gssapi_ext.h>
 # endif
@@ -166,7 +166,7 @@ namespace shibsp {
         long sendResponse(std::istream& inputStream, long status);
         
         // HTTPResponse
-        void setResponseHeader(const char* name, const char* value);
+        void setResponseHeader(const char* name, const char* value, bool replace=false);
         long sendRedirect(const char* url);
     };
 }
@@ -210,10 +210,10 @@ const std::vector<XSECCryptoX509*>& RemotedRequest::getClientCertificates() cons
             }
             catch(XSECException& e) {
                 auto_ptr_char temp(e.getMsg());
-                Category::getInstance(SHIBSP_LOGCAT".SPRequest").error("XML-Security exception loading client certificate: %s", temp.get());
+                Category::getInstance(SHIBSP_LOGCAT ".SPRequest").error("XML-Security exception loading client certificate: %s", temp.get());
             }
             catch(XSECCryptoException& e) {
-                Category::getInstance(SHIBSP_LOGCAT".SPRequest").error("XML-Security exception loading client certificate: %s", e.getMsg());
+                Category::getInstance(SHIBSP_LOGCAT ".SPRequest").error("XML-Security exception loading client certificate: %s", e.getMsg());
             }
             cert = certs.next();
         }
@@ -227,7 +227,7 @@ gss_ctx_id_t RemotedRequest::getGSSContext() const
     if (m_gssctx == GSS_C_NO_CONTEXT) {
         const char* encoded = m_input["gss_context"].string();
         if (encoded) {
-            xsecsize_t x;
+            XMLSize_t x;
             XMLByte* decoded = Base64::decode(reinterpret_cast<const XMLByte*>(encoded), &x);
             if (decoded) {
                 gss_buffer_desc importbuf;
@@ -237,11 +237,7 @@ gss_ctx_id_t RemotedRequest::getGSSContext() const
                 OM_uint32 major = gss_import_sec_context(&minor, &importbuf, &m_gssctx);
                 if (major != GSS_S_COMPLETE)
                     m_gssctx = GSS_C_NO_CONTEXT;
-#ifdef SHIBSP_XERCESC_HAS_XMLBYTE_RELEASE
-                XMLString::release(&decoded);
-#else
                 XMLString::release((char**)&decoded);
-#endif
             }
         }
     }
@@ -253,24 +249,20 @@ gss_name_t RemotedRequest::getGSSName() const
     if (m_gssname == GSS_C_NO_NAME) {
         const char* encoded = m_input["gss_name"].string();
         if (encoded) {
-            xsecsize_t x;
+            XMLSize_t x;
             XMLByte* decoded = Base64::decode(reinterpret_cast<const XMLByte*>(encoded), &x);
             gss_buffer_desc importbuf;
             importbuf.length = x;
             importbuf.value = decoded;
             OM_uint32 major,minor;
-#ifdef HAVE_GSSAPI_COMPOSITE_NAME
+#if HAVE_DECL_GSS_C_NT_EXPORT_NAME_COMPOSITE
             major = gss_import_name(&minor, &importbuf, GSS_C_NT_EXPORT_NAME_COMPOSITE, &m_gssname);
 #else
             major = gss_import_name(&minor, &importbuf, GSS_C_NT_EXPORT_NAME, &m_gssname);
 #endif
             if (major != GSS_S_COMPLETE)
                 m_gssname = GSS_C_NO_NAME;
-#ifdef SHIBSP_XERCESC_HAS_XMLBYTE_RELEASE
-            XMLString::release(&decoded);
-#else
             XMLString::release((char**)&decoded);
-#endif
         }
 
         if (m_gssname == GSS_C_NO_NAME) {
@@ -297,20 +289,33 @@ long RemotedResponse::sendResponse(std::istream& in, long status)
     }
     if (!m_output.isstruct())
         m_output.structure();
-    m_output.addmember("response.data").string(msg.c_str());
+    m_output.addmember("response.data").unsafe_string(msg.c_str());
     m_output.addmember("response.status").integer(status);
     return status;
 }
 
-void RemotedResponse::setResponseHeader(const char* name, const char* value)
+void RemotedResponse::setResponseHeader(const char* name, const char* value, bool replace)
 {
+    HTTPResponse::setResponseHeader(name, value, replace);
+
     if (!m_output.isstruct())
         m_output.structure();
     DDF hdrs = m_output["headers"];
     if (hdrs.isnull())
         hdrs = m_output.addmember("headers").list();
-    DDF h = DDF(name).string(value);
-    hdrs.add(h);
+    if (replace || !value) {
+        DDF hdr = hdrs.first();
+        while (!hdr.isnull()) {
+            if (hdr.name() && !strcmp(hdr.name(), name))
+                hdr.destroy();
+            hdr = hdrs.next();
+        }
+    }
+
+    if (value && *value) {
+        DDF h = DDF(name).unsafe_string(value);
+        hdrs.add(h);
+    }
 }
 
 long RemotedResponse::sendRedirect(const char* url)
@@ -329,13 +334,8 @@ void RemotedHandler::setAddress(const char* address)
         throw ConfigurationException("Cannot register a remoting address twice for the same Handler.");
     m_address = address;
     SPConfig& conf = SPConfig::getConfig();
-    if (!conf.isEnabled(SPConfig::InProcess)) {
-        ListenerService* listener = conf.getServiceProvider()->getListenerService(false);
-        if (listener)
-            listener->regListener(m_address.c_str(), this);
-        else
-            Category::getInstance(SHIBSP_LOGCAT".Handler").info("no ListenerService available, handler remoting disabled");
-    }
+    if (conf.isEnabled(SPConfig::OutOfProcess) && !conf.isEnabled(SPConfig::InProcess))
+        conf.getServiceProvider()->regListener(address, this);
 }
 
 set<string> RemotedHandler::m_remotedHeaders;
@@ -347,9 +347,8 @@ RemotedHandler::RemotedHandler()
 RemotedHandler::~RemotedHandler()
 {
     SPConfig& conf = SPConfig::getConfig();
-    ListenerService* listener=conf.getServiceProvider()->getListenerService(false);
-    if (listener && conf.isEnabled(SPConfig::OutOfProcess) && !conf.isEnabled(SPConfig::InProcess))
-        listener->unregListener(m_address.c_str(),this);
+    if (conf.isEnabled(SPConfig::OutOfProcess) && !conf.isEnabled(SPConfig::InProcess))
+        conf.getServiceProvider()->unregListener(m_address.c_str(), this);
 }
 
 void RemotedHandler::addRemotedHeader(const char* header)
@@ -357,6 +356,21 @@ void RemotedHandler::addRemotedHeader(const char* header)
     m_remotedHeaders.insert(header);
 }
 
+DDF RemotedHandler::send(const SPRequest& request, DDF& in) const
+{
+    // Capture and forward entityIDSelf content setting, if set.
+    pair<bool, const char*> entityID = request.getRequestSettings().first->getString("entityIDSelf");
+    if (entityID.first) {
+        string s(entityID.second);
+        string::size_type pos = s.find("$hostname");
+        if (pos != string::npos)
+            s.replace(pos, 9, request.getHostname());
+        in.addmember("_mapped.entityID").string(s.c_str());
+    }
+
+    return request.getServiceProvider().getListenerService()->send(in);
+}
+
 DDF RemotedHandler::wrap(const SPRequest& request, const vector<string>* headers, bool certs) const
 {
     DDF in = DDF(m_address.c_str()).structure();
@@ -422,17 +436,13 @@ DDF RemotedHandler::wrap(const SPRequest& request, const vector<string>* headers
             gss_buffer_desc contextbuf = GSS_C_EMPTY_BUFFER;
             OM_uint32 major = gss_export_sec_context(&minor, &ctx, &contextbuf);
             if (major == GSS_S_COMPLETE) {
-                xsecsize_t len = 0;
+                XMLSize_t len = 0;
                 XMLByte* out = Base64::encode(reinterpret_cast<const XMLByte*>(contextbuf.value), contextbuf.length, &len);
                 gss_release_buffer(&minor, &contextbuf);
                 if (out) {
                     string ctx;
                     ctx.append(reinterpret_cast<char*>(out), len);
-#ifdef SHIBSP_XERCESC_HAS_XMLBYTE_RELEASE
-                    XMLString::release(&out);
-#else
                     XMLString::release((char**)&out);
-#endif
                     in.addmember("gss_context").string(ctx.c_str());
                 }
                 else {
@@ -443,7 +453,7 @@ DDF RemotedHandler::wrap(const SPRequest& request, const vector<string>* headers
                 request.log(SPRequest::SPError, "error while exporting GSS context");
             }
         }
-#ifdef HAVE_GSSAPI_NAMINGEXTS
+#if HAVE_DECL_GSS_GET_NAME_ATTRIBUTE
         else {
             gss_name_t name = gss->getGSSName();
             if (name != GSS_C_NO_NAME) {
@@ -451,17 +461,13 @@ DDF RemotedHandler::wrap(const SPRequest& request, const vector<string>* headers
                 gss_buffer_desc namebuf = GSS_C_EMPTY_BUFFER;
                 OM_uint32 major = gss_export_name_composite(&minor, name, &namebuf);
                 if (major == GSS_S_COMPLETE) {
-                    xsecsize_t len = 0;
+                    XMLSize_t len = 0;
                     XMLByte* out = Base64::encode(reinterpret_cast<const XMLByte*>(namebuf.value), namebuf.length, &len);
                     gss_release_buffer(&minor, &namebuf);
                     if (out) {
                         string nm;
                         nm.append(reinterpret_cast<char*>(out), len);
-#ifdef SHIBSP_XERCESC_HAS_XMLBYTE_RELEASE
-                        XMLString::release(&out);
-#else
                         XMLString::release((char**)&out);
-#endif
                         in.addmember("gss_name").string(nm.c_str());
                     }
                     else {
@@ -500,8 +506,11 @@ pair<bool,long> RemotedHandler::unwrap(SPRequest& request, DDF& out) const
         return make_pair(true, request.sendRedirect(h.string()));
     h = out["response"];
     if (h.isstruct()) {
-        istringstream s(h["data"].string());
-        return make_pair(true, request.sendResponse(s, h["status"].integer()));
+        const char* data = h["data"].string();
+        if (data) {
+            istringstream s(data);
+            return make_pair(true, request.sendResponse(s, h["status"].integer()));
+        }
     }
     return make_pair(false, 0L);
 }
