@@ -73,7 +73,7 @@ namespace shibsp {
     class SHIBSP_API SAML2ArtifactResolution : public AbstractHandler, public RemotedHandler 
     {
     public:
-        SAML2ArtifactResolution(const DOMElement* e, const char* appId);
+        SAML2ArtifactResolution(const DOMElement* e, const char* appId, bool deprecationSupport=true);
         virtual ~SAML2ArtifactResolution() {}
 
         pair<bool,long> run(SPRequest& request, bool isHandler=true) const;
@@ -88,7 +88,7 @@ namespace shibsp {
             // Initial guess at index to use.
             pair<bool,unsigned int> ix = pair<bool,unsigned int>(false,0);
             if (!strncmp(handlerURL, "https", 5))
-                ix = getUnsignedInt("sslIndex", shibspconstants::ASCII_SHIB2SPCONFIG_NS);
+                ix = getUnsignedInt("sslIndex", shibspconstants::ASCII_SHIBSPCONFIG_NS);
             if (!ix.first)
                 ix = getUnsignedInt("index");
             if (!ix.first)
@@ -134,28 +134,20 @@ namespace shibsp {
     #pragma warning( pop )
 #endif
 
-    Handler* SHIBSP_DLLLOCAL SAML2ArtifactResolutionFactory(const pair<const DOMElement*,const char*>& p)
+    Handler* SHIBSP_DLLLOCAL SAML2ArtifactResolutionFactory(const pair<const DOMElement*,const char*>& p, bool deprecationSupport)
     {
-        return new SAML2ArtifactResolution(p.first, p.second);
+        return new SAML2ArtifactResolution(p.first, p.second, deprecationSupport);
     }
 
 };
 
-SAML2ArtifactResolution::SAML2ArtifactResolution(const DOMElement* e, const char* appId)
+SAML2ArtifactResolution::SAML2ArtifactResolution(const DOMElement* e, const char* appId, bool deprecationSupport)
     : AbstractHandler(e, Category::getInstance(SHIBSP_LOGCAT ".ArtifactResolution.SAML2"))
 {
 #ifndef SHIBSP_LITE
     if (SPConfig::getConfig().isEnabled(SPConfig::OutOfProcess)) {
-        m_encoder.reset(
-            SAMLConfig::getConfig().MessageEncoderManager.newPlugin(
-                getString("Binding").second, pair<const DOMElement*,const XMLCh*>(e,shibspconstants::SHIB2SPCONFIG_NS)
-                )
-            );
-        m_decoder.reset(
-            SAMLConfig::getConfig().MessageDecoderManager.newPlugin(
-                getString("Binding").second, pair<const DOMElement*,const XMLCh*>(e,shibspconstants::SHIB2SPCONFIG_NS)
-                )
-            );
+        m_encoder.reset(SAMLConfig::getConfig().MessageEncoderManager.newPlugin(getString("Binding").second, e, deprecationSupport));
+        m_decoder.reset(SAMLConfig::getConfig().MessageDecoderManager.newPlugin(getString("Binding").second, e, deprecationSupport));
     }
 #endif
     string address(appId);
@@ -179,7 +171,7 @@ pair<bool,long> SAML2ArtifactResolution::run(SPRequest& request, bool isHandler)
             DDF out,in = wrap(request, nullptr, true);
             DDFJanitor jin(in), jout(out);
             
-            out = request.getServiceProvider().getListenerService()->send(in);
+            out = send(request, in);
             return unwrap(request, out);
         }
     }
@@ -193,7 +185,7 @@ pair<bool,long> SAML2ArtifactResolution::run(SPRequest& request, bool isHandler)
         code->setCode(&Faultcode::SERVER);
         Faultstring* fs = FaultstringBuilder::buildFaultstring();
         fault->setFaultstring(fs);
-        pair<bool,bool> flag = getBool("detailedErrors", m_configNS.get());
+        pair<bool,bool> flag = getBool("detailedErrors", shibspconstants::ASCII_SHIBSPCONFIG_NS);
         auto_ptr_XMLCh msg((flag.first && flag.second) ? ex.what() : "Error processing request.");
         fs->setString(msg.get());
 #ifndef SHIBSP_LITE
@@ -255,7 +247,7 @@ void SAML2ArtifactResolution::receive(DDF& in, ostream& out)
         code->setCode(&Faultcode::SERVER);
         Faultstring* fs = FaultstringBuilder::buildFaultstring();
         fault->setFaultstring(fs);
-        pair<bool,bool> flag = getBool("detailedErrors", m_configNS.get());
+        pair<bool,bool> flag = getBool("detailedErrors", shibspconstants::ASCII_SHIBSPCONFIG_NS);
         auto_ptr_XMLCh msg((flag.first && flag.second) ? ex.what() : "Error processing request.");
         fs->setString(msg.get());
         m_encoder->encode(*resp, fault.get(), nullptr);
@@ -277,7 +269,7 @@ pair<bool,long> SAML2ArtifactResolution::processMessage(const Application& appli
         throw ConfigurationException("No ArtifactMap instance installed.");
 
     // Locate policy key.
-    pair<bool,const char*> policyId = getString("policyId", m_configNS.get());  // namespace-qualified if inside handler element
+    pair<bool,const char*> policyId = getString("policyId", shibspconstants::ASCII_SHIBSPCONFIG_NS);  // namespace-qualified if inside handler element
     if (!policyId.first)
         policyId = application.getString("policyId");   // unqualified in Application(s) element
         
@@ -323,12 +315,14 @@ pair<bool,long> SAML2ArtifactResolution::processMessage(const Application& appli
         auto_ptr<ArtifactResponse> resp(ArtifactResponseBuilder::buildArtifactResponse());
         resp->setInResponseTo(req->getID());
         Issuer* me = IssuerBuilder::buildIssuer();
+        resp->setIssuer(me);
         me->setName(application.getRelyingParty(entity)->getXMLString("entityID").second);
         resp->setPayload(payload.get());
         payload.release();
+        fillStatus(*resp, StatusCode::SUCCESS);
 
         long ret = sendMessage(
-            *m_encoder, resp.get(), relayState.c_str(), nullptr, policy->getIssuerMetadata(), application, httpResponse, "signResponses"
+            *m_encoder, resp.get(), relayState.c_str(), nullptr, policy->getIssuerMetadata(), application, httpResponse, "conditional"
             );
         resp.release();  // freed by encoder
         return make_pair(true, ret);
@@ -351,6 +345,7 @@ pair<bool,long> SAML2ArtifactResolution::emptyResponse(
     auto_ptr<ArtifactResponse> resp(ArtifactResponseBuilder::buildArtifactResponse());
     resp->setInResponseTo(request.getID());
     Issuer* me = IssuerBuilder::buildIssuer();
+    resp->setIssuer(me);
     me->setName(app.getRelyingParty(recipient)->getXMLString("entityID").second);
     fillStatus(*resp, StatusCode::SUCCESS);
     long ret = m_encoder->encode(httpResponse, resp.get(), nullptr);
