@@ -29,6 +29,7 @@
 #include "Application.h"
 #include "ServiceProvider.h"
 #include "SPRequest.h"
+#include "TransactionLog.h"
 #include "handler/AbstractHandler.h"
 #include "handler/RemotedHandler.h"
 #include "util/SPConstants.h"
@@ -72,7 +73,7 @@ namespace shibsp {
     class SHIBSP_DLLLOCAL SAML2NameIDMgmt : public AbstractHandler, public RemotedHandler
     {
     public:
-        SAML2NameIDMgmt(const DOMElement* e, const char* appId);
+        SAML2NameIDMgmt(const DOMElement* e, const char* appId, bool deprecationSupport=true);
         virtual ~SAML2NameIDMgmt() {}
 
         void receive(DDF& in, ostream& out);
@@ -101,8 +102,12 @@ namespace shibsp {
             return samlconstants::SAML20P_NS;
         }
 
+        const char* getEventType() const {
+            return NAMEIDMGMT_EVENT;
+        }
+
     private:
-        pair<bool,long> doRequest(const Application& application, const HTTPRequest& httpRequest, HTTPResponse& httpResponse) const;
+        pair<bool,long> doRequest(const Application& application, HTTPRequest& httpRequest, HTTPResponse& httpResponse) const;
 
 #ifndef SHIBSP_LITE
         bool notifyBackChannel(const Application& application, const char* requestURL, const NameID& nameid, const NewID* newid) const;
@@ -129,13 +134,13 @@ namespace shibsp {
     #pragma warning( pop )
 #endif
 
-    Handler* SHIBSP_DLLLOCAL SAML2NameIDMgmtFactory(const pair<const DOMElement*,const char*>& p)
+    Handler* SHIBSP_DLLLOCAL SAML2NameIDMgmtFactory(const pair<const DOMElement*,const char*>& p, bool deprecationSupport)
     {
-        return new SAML2NameIDMgmt(p.first, p.second);
+        return new SAML2NameIDMgmt(p.first, p.second, deprecationSupport);
     }
 };
 
-SAML2NameIDMgmt::SAML2NameIDMgmt(const DOMElement* e, const char* appId)
+SAML2NameIDMgmt::SAML2NameIDMgmt(const DOMElement* e, const char* appId, bool deprecationSupport)
     : AbstractHandler(e, Category::getInstance(SHIBSP_LOGCAT ".NameIDMgmt.SAML2"))
 {
 #ifndef SHIBSP_LITE
@@ -143,17 +148,13 @@ SAML2NameIDMgmt::SAML2NameIDMgmt(const DOMElement* e, const char* appId)
         SAMLConfig& conf = SAMLConfig::getConfig();
 
         // Handle incoming binding.
-        m_decoder.reset(
-            conf.MessageDecoderManager.newPlugin(
-                getString("Binding").second, pair<const DOMElement*,const XMLCh*>(e,shibspconstants::SHIB2SPCONFIG_NS)
-                )
-            );
+        m_decoder.reset(conf.MessageDecoderManager.newPlugin(getString("Binding").second, e, deprecationSupport));
         m_decoder->setArtifactResolver(SPConfig::getConfig().getArtifactResolver());
 
         if (m_decoder->isUserAgentPresent()) {
             // Handle front-channel binding setup.
             string dupBindings;
-            pair<bool,const char*> outgoing = getString("outgoingBindings", m_configNS.get());
+            pair<bool,const char*> outgoing = getString("outgoingBindings", shibspconstants::ASCII_SHIBSPCONFIG_NS);
             if (outgoing.first) {
                 dupBindings = outgoing.second;
                 trim(dupBindings);
@@ -167,9 +168,7 @@ SAML2NameIDMgmt::SAML2NameIDMgmt(const DOMElement* e, const char* appId)
             split(m_bindings, dupBindings, is_space(), algorithm::token_compress_on);
             for (vector<string>::const_iterator b = m_bindings.begin(); b != m_bindings.end(); ++b) {
                 try {
-                    boost::shared_ptr<MessageEncoder> encoder(
-                        conf.MessageEncoderManager.newPlugin(*b, pair<const DOMElement*,const XMLCh*>(e,shibspconstants::SHIB2SPCONFIG_NS))
-                        );
+                    boost::shared_ptr<MessageEncoder> encoder(conf.MessageEncoderManager.newPlugin(*b, e, deprecationSupport));
                     if (encoder->isUserAgentPresent() && XMLString::equals(getProtocolFamily(), encoder->getProtocolFamily())) {
                         m_encoders[*b] = encoder;
                         m_log.debug("supporting outgoing binding (%s)", b->c_str());
@@ -185,9 +184,7 @@ SAML2NameIDMgmt::SAML2NameIDMgmt(const DOMElement* e, const char* appId)
         }
         else {
             pair<bool,const char*> b = getString("Binding");
-            boost::shared_ptr<MessageEncoder> encoder(
-                conf.MessageEncoderManager.newPlugin(b.second, pair<const DOMElement*,const XMLCh*>(e,shibspconstants::SHIB2SPCONFIG_NS))
-                );
+            boost::shared_ptr<MessageEncoder> encoder(conf.MessageEncoderManager.newPlugin(b.second, e, deprecationSupport));
             m_encoders[b.second] = encoder;
         }
     }
@@ -211,7 +208,7 @@ pair<bool,long> SAML2NameIDMgmt::run(SPRequest& request, bool isHandler) const
         headers.push_back("User-Agent");
         DDF out,in = wrap(request, &headers, true);
         DDFJanitor jin(in), jout(out);
-        out=request.getServiceProvider().getListenerService()->send(in);
+        out = send(request, in);
         return unwrap(request, out);
     }
 }
@@ -242,13 +239,13 @@ void SAML2NameIDMgmt::receive(DDF& in, ostream& out)
     out << ret;
 }
 
-pair<bool,long> SAML2NameIDMgmt::doRequest(const Application& application, const HTTPRequest& request, HTTPResponse& response) const
+pair<bool,long> SAML2NameIDMgmt::doRequest(const Application& application, HTTPRequest& request, HTTPResponse& response) const
 {
 #ifndef SHIBSP_LITE
     SessionCache* cache = application.getServiceProvider().getSessionCache();
 
     // Locate policy key.
-    pair<bool,const char*> policyId = getString("policyId", m_configNS.get());  // may be namespace-qualified inside handler element
+    pair<bool,const char*> policyId = getString("policyId", shibspconstants::ASCII_SHIBSPCONFIG_NS);  // may be namespace-qualified inside handler element
     if (!policyId.first)
         policyId = getString("policyId");   // try unqualified
     if (!policyId.first)
@@ -512,7 +509,7 @@ pair<bool,long> SAML2NameIDMgmt::sendResponse(
 
     auto_ptr_char dest(nim->getDestination());
 
-    long ret = sendMessage(*encoder, nim.get(), relayState, dest.get(), role, application, httpResponse);
+    long ret = sendMessage(*encoder, nim.get(), relayState, dest.get(), role, application, httpResponse, "conditional");
     nim.release();  // freed by encoder
     return make_pair(true, ret);
 }
