@@ -27,6 +27,7 @@
 #include "internal.h"
 #include "exceptions.h"
 #include "SPRequest.h"
+#include "TransactionLog.h"
 #include "handler/SessionInitiator.h"
 
 using namespace shibsp;
@@ -49,8 +50,6 @@ namespace shibsp {
     SHIBSP_DLLLOCAL PluginManager< SessionInitiator,string,pair<const DOMElement*,const char*> >::Factory CookieSessionInitiatorFactory;
 };
 
-map<string,string> SessionInitiator::m_remapper;
-
 void SHIBSP_API shibsp::registerSessionInitiators()
 {
     SPConfig& conf=SPConfig::getConfig();
@@ -62,8 +61,6 @@ void SHIBSP_API shibsp::registerSessionInitiators()
     conf.SessionInitiatorManager.registerFactory(TRANSFORM_SESSION_INITIATOR, TransformSessionInitiatorFactory);
     conf.SessionInitiatorManager.registerFactory(FORM_SESSION_INITIATOR, FormSessionInitiatorFactory);
     conf.SessionInitiatorManager.registerFactory(COOKIE_SESSION_INITIATOR, CookieSessionInitiatorFactory);
-
-    SessionInitiator::m_remapper["defaultACSIndex"] = "acsIndex";
 }
 
 SessionInitiator::SessionInitiator()
@@ -74,6 +71,21 @@ SessionInitiator::~SessionInitiator()
 {
 }
 
+const char* SessionInitiator::remap(const char* src, Category& log) const
+{
+    if (XMLString::equals(src, "defaultACSIndex")) {
+        log.warn("DEPRECATED configuration - remapping property/set (%s) to (%s)", src, "acsIndex");
+        return "acsIndex";
+    }
+    else {
+        return src;
+    }
+}
+
+const char* SessionInitiator::getEventType() const {
+    return LOGIN_EVENT;
+}
+
 #ifndef SHIBSP_LITE
 const char* SessionInitiator::getType() const
 {
@@ -216,7 +228,7 @@ AuthnRequestEvent* SessionInitiator::newAuthnRequestEvent(const Application& app
     if (!SPConfig::getConfig().isEnabled(SPConfig::Logging))
         return nullptr;
     try {
-        auto_ptr<TransactionLog::Event> event(SPConfig::getConfig().EventManager.newPlugin(AUTHNREQUEST_EVENT, nullptr));
+        auto_ptr<TransactionLog::Event> event(SPConfig::getConfig().EventManager.newPlugin(AUTHNREQUEST_EVENT, nullptr, false));
         AuthnRequestEvent* ar_event = dynamic_cast<AuthnRequestEvent*>(event.get());
         if (ar_event) {
             ar_event->m_request = request;
