@@ -36,7 +36,7 @@
 #include <boost/iterator/indirect_iterator.hpp>
 #include <boost/scoped_ptr.hpp>
 #include <xmltooling/version.h>
-#include <xmltooling/util/DateTime.h>
+#include <xercesc/util/XMLDateTime.hpp>
 
 #ifdef HAVE_SYS_UTSNAME_H
 # include <sys/utsname.h>
@@ -83,7 +83,7 @@ namespace shibsp {
     #pragma warning( pop )
 #endif
 
-    Handler* SHIBSP_DLLLOCAL StatusHandlerFactory(const pair<const DOMElement*,const char*>& p)
+    Handler* SHIBSP_DLLLOCAL StatusHandlerFactory(const pair<const DOMElement*,const char*>& p, bool)
     {
         return new StatusHandler(p.first, p.second);
     }
@@ -251,7 +251,7 @@ namespace shibsp {
 };
 
 StatusHandler::StatusHandler(const DOMElement* e, const char* appId)
-    : SecuredHandler(e, Category::getInstance(SHIBSP_LOGCAT".StatusHandler"))
+    : SecuredHandler(e, Category::getInstance(SHIBSP_LOGCAT ".Handler.Status"))
 {
     string address(appId);
     address += getString("Location").second;
@@ -270,10 +270,7 @@ pair<bool,long> StatusHandler::run(SPRequest& request, bool isHandler) const
         // RequestMap query, so handle it inproc.
         DummyRequest dummy(target);
         RequestMapper::Settings settings = request.getApplication().getServiceProvider().getRequestMapper()->getSettings(dummy);
-        map<string,const char*> props;
-        settings.first->getAll(props);
-
-        DateTime now(time(nullptr));
+        XMLDateTime now(time(nullptr), false);
         now.parseDateTime();
         auto_ptr_char timestamp(now.getFormattedString());
         request.setContentType("text/xml");
@@ -286,10 +283,14 @@ pair<bool,long> StatusHandler::run(SPRequest& request, bool isHandler) const
                 << "' OpenSAML-C='" << gOpenSAMLDotVersionStr
 #endif
                 << "' Shibboleth='" << PACKAGE_VERSION << "'/>";
-            systemInfo(msg) << "<RequestSettings";
-            for (map<string,const char*>::const_iterator p = props.begin(); p != props.end(); ++p)
-                msg << ' ' << p->first << "='" << p->second << "'";
-            msg << '>' << target << "</RequestSettings>";
+            const char* setting = request.getParameter("setting");
+                systemInfo(msg) << "<RequestSettings";
+                if (setting) {
+                    pair<bool, const char*> prop = settings.first->getString(setting);
+                    if (prop.first)
+                        msg << ' ' << setting << "='" << prop.second << "'";
+                }
+                msg << '>' << target << "</RequestSettings>";
             msg << "<Status><OK/></Status>";
         msg << "</StatusHandler>";
         return make_pair(true, request.sendResponse(msg));
@@ -304,13 +305,13 @@ pair<bool,long> StatusHandler::run(SPRequest& request, bool isHandler) const
             // When not out of process, we remote all the message processing.
             DDF out,in = wrap(request);
             DDFJanitor jin(in), jout(out);
-            out=request.getServiceProvider().getListenerService()->send(in);
+            out = send(request, in);
             return unwrap(request, out);
         }
     }
     catch (XMLToolingException& ex) {
         m_log.error("error while processing request: %s", ex.what());
-        DateTime now(time(nullptr));
+        XMLDateTime now(time(nullptr), false);
         now.parseDateTime();
         auto_ptr_char timestamp(now.getFormattedString());
         request.setContentType("text/xml");
@@ -329,7 +330,7 @@ pair<bool,long> StatusHandler::run(SPRequest& request, bool isHandler) const
     }
     catch (std::exception& ex) {
         m_log.error("error while processing request: %s", ex.what());
-        DateTime now(time(nullptr));
+        XMLDateTime now(time(nullptr), false);
         now.parseDateTime();
         auto_ptr_char timestamp(now.getFormattedString());
         request.setContentType("text/xml");
@@ -379,7 +380,7 @@ pair<bool,long> StatusHandler::processMessage(
 #ifndef SHIBSP_LITE
     m_log.debug("processing status request");
 
-    DateTime now(time(nullptr));
+    XMLDateTime now(time(nullptr), false);
     now.parseDateTime();
     auto_ptr_char timestamp(now.getFormattedString());
 
