@@ -110,8 +110,8 @@ namespace shibsp {
     class XMLAccessControl : public AccessControl, public ReloadableXMLFile
     {
     public:
-        XMLAccessControl(const DOMElement* e)
-                : ReloadableXMLFile(e, Category::getInstance(SHIBSP_LOGCAT ".AccessControl.XML")) {
+        XMLAccessControl(const DOMElement* e, bool deprecationSupport=true)
+                : ReloadableXMLFile(e, Category::getInstance(SHIBSP_LOGCAT ".AccessControl.XML"), true, deprecationSupport) {
             background_load(); // guarantees an exception or the policy is loaded
         }
 
@@ -132,22 +132,21 @@ namespace shibsp {
     #pragma warning( pop )
 #endif
 
-    AccessControl* SHIBSP_DLLLOCAL XMLAccessControlFactory(const DOMElement* const & e)
+    AccessControl* SHIBSP_DLLLOCAL XMLAccessControlFactory(const DOMElement* const & e, bool deprecationSupport)
     {
-        return new XMLAccessControl(e);
+        return new XMLAccessControl(e, deprecationSupport);
     }
 
-    static const XMLCh _AccessControl[] =   UNICODE_LITERAL_13(A,c,c,e,s,s,C,o,n,t,r,o,l);
-    static const XMLCh _Handler[] =         UNICODE_LITERAL_7(H,a,n,d,l,e,r);
-    static const XMLCh ignoreCase[] =       UNICODE_LITERAL_10(i,g,n,o,r,e,C,a,s,e);
-    static const XMLCh ignoreOption[] =     UNICODE_LITERAL_1(i);
-    static const XMLCh _list[] =            UNICODE_LITERAL_4(l,i,s,t);
-    static const XMLCh require[] =          UNICODE_LITERAL_7(r,e,q,u,i,r,e);
-    static const XMLCh NOT[] =              UNICODE_LITERAL_3(N,O,T);
-    static const XMLCh AND[] =              UNICODE_LITERAL_3(A,N,D);
-    static const XMLCh OR[] =               UNICODE_LITERAL_2(O,R);
-    static const XMLCh _Rule[] =            UNICODE_LITERAL_4(R,u,l,e);
-    static const XMLCh _RuleRegex[] =       UNICODE_LITERAL_9(R,u,l,e,R,e,g,e,x);
+    static const XMLCh _AccessControl[] =        UNICODE_LITERAL_13(A,c,c,e,s,s,C,o,n,t,r,o,l);
+    static const XMLCh _Handler[] =              UNICODE_LITERAL_7(H,a,n,d,l,e,r);
+    static const XMLCh caseInsensitiveOption[] = UNICODE_LITERAL_1(i);
+    static const XMLCh _list[] =                 UNICODE_LITERAL_4(l,i,s,t);
+    static const XMLCh require[] =               UNICODE_LITERAL_7(r,e,q,u,i,r,e);
+    static const XMLCh NOT[] =                   UNICODE_LITERAL_3(N,O,T);
+    static const XMLCh AND[] =                   UNICODE_LITERAL_3(A,N,D);
+    static const XMLCh OR[] =                    UNICODE_LITERAL_2(O,R);
+    static const XMLCh _Rule[] =                 UNICODE_LITERAL_4(R,u,l,e);
+    static const XMLCh _RuleRegex[] =            UNICODE_LITERAL_9(R,u,l,e,R,e,g,e,x);
 }
 
 Rule::Rule(const DOMElement* e) : m_alias(XMLHelper::getAttrString(e, nullptr, require))
@@ -157,7 +156,7 @@ Rule::Rule(const DOMElement* e) : m_alias(XMLHelper::getAttrString(e, nullptr, r
     if (!e->hasChildNodes())
         return; // empty rule
 
-    auto_arrayptr<char> vals(toUTF8(e->getTextContent()));
+    auto_arrayptr<char> vals(toUTF8(XMLHelper::getTextContent(e)));
     if (!vals.get() || !*vals.get())
         throw ConfigurationException("Unable to convert Rule content into UTF-8.");
 
@@ -253,9 +252,9 @@ RuleRegex::RuleRegex(const DOMElement* e)
     if (m_alias.empty() || !m_exp.get() || !*m_exp.get())
         throw ConfigurationException("Access control rule missing require attribute or element content.");
 
-    bool ignore = XMLHelper::getAttrBool(e, false, ignoreCase);
+    bool caseSensitive = XMLHelper::getCaseSensitive(e, true);
     try {
-        m_re.reset(new RegularExpression(e->getFirstChild()->getNodeValue(), (ignore ? ignoreOption : &chNull)));
+        m_re.reset(new RegularExpression(e->getFirstChild()->getNodeValue(), (caseSensitive ? &chNull: caseInsensitiveOption )));
     }
     catch (XMLException& ex) {
         auto_ptr_char tmp(ex.getMessage());
