@@ -45,7 +45,6 @@
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/regx/RegularExpression.hpp>
 
-using shibspconstants::SHIB2SPCONFIG_NS;
 using namespace shibsp;
 using namespace xmltooling;
 using namespace boost;
@@ -76,12 +75,7 @@ namespace shibsp {
         ~Override() {}
 
         // Provides filter to exclude special config elements.
-#ifdef SHIBSP_XERCESC_SHORT_ACCEPTNODE
-        short
-#else
-        FilterAction
-#endif
-        acceptNode(const DOMNode* node) const {
+        FilterAction acceptNode(const DOMNode* node) const {
             return FILTER_REJECT;
         }
 
@@ -94,7 +88,7 @@ namespace shibsp {
         bool m_unicodeAware;
         map< string,boost::shared_ptr<Override> > m_map;
         vector< pair< boost::shared_ptr<RegularExpression>,boost::shared_ptr<Override> > > m_regexps;
-        vector< tuple< string,boost::shared_ptr<RegularExpression>,boost::shared_ptr<Override> > > m_queries;
+        vector< boost::tuple< string,boost::shared_ptr<RegularExpression>,boost::shared_ptr<Override> > > m_queries;
 
     private:
         scoped_ptr<AccessControl> m_acl;
@@ -128,7 +122,8 @@ namespace shibsp {
     class XMLRequestMapper : public RequestMapper, public ReloadableXMLFile
     {
     public:
-        XMLRequestMapper(const DOMElement* e) : ReloadableXMLFile(e,Category::getInstance(SHIBSP_LOGCAT".RequestMapper")) {
+        XMLRequestMapper(const DOMElement* e, bool deprecationSupport=true)
+            : ReloadableXMLFile(e, Category::getInstance(SHIBSP_LOGCAT ".RequestMapper"), true, deprecationSupport) {
             background_load();
         }
 
@@ -149,18 +144,18 @@ namespace shibsp {
     #pragma warning( pop )
 #endif
 
-    RequestMapper* SHIBSP_DLLLOCAL XMLRequestMapperFactory(const DOMElement* const & e)
+    RequestMapper* SHIBSP_DLLLOCAL XMLRequestMapperFactory(const DOMElement* const & e, bool deprecationSupport)
     {
-        return new XMLRequestMapper(e);
+        return new XMLRequestMapper(e, deprecationSupport);
     }
 
     static const XMLCh _AccessControl[] =           UNICODE_LITERAL_13(A,c,c,e,s,s,C,o,n,t,r,o,l);
     static const XMLCh AccessControlProvider[] =    UNICODE_LITERAL_21(A,c,c,e,s,s,C,o,n,t,r,o,l,P,r,o,v,i,d,e,r);
+    static const XMLCh caseInsensitiveOption[] =    UNICODE_LITERAL_1(i);
     static const XMLCh Host[] =                     UNICODE_LITERAL_4(H,o,s,t);
     static const XMLCh HostRegex[] =                UNICODE_LITERAL_9(H,o,s,t,R,e,g,e,x);
     static const XMLCh htaccess[] =                 UNICODE_LITERAL_8(h,t,a,c,c,e,s,s);
     static const XMLCh ignoreCase[] =               UNICODE_LITERAL_10(i,g,n,o,r,e,C,a,s,e);
-    static const XMLCh ignoreOption[] =             UNICODE_LITERAL_1(i);
     static const XMLCh Path[] =                     UNICODE_LITERAL_4(P,a,t,h);
     static const XMLCh PathRegex[] =                UNICODE_LITERAL_9(P,a,t,h,R,e,g,e,x);
     static const XMLCh Query[] =                    UNICODE_LITERAL_5(Q,u,e,r,y);
@@ -186,17 +181,18 @@ RequestMapper::~RequestMapper()
 
 void Override::loadACL(const DOMElement* e, Category& log)
 {
+    bool deprecationSupport = e ? XMLString::equals(e->getNamespaceURI(), shibspconstants::SHIB2SPCONFIG_NS) : false;
     try {
         const DOMElement* acl = XMLHelper::getFirstChildElement(e,htaccess);
         if (acl) {
             log.info("building Apache htaccess AccessControl provider...");
-            m_acl.reset(SPConfig::getConfig().AccessControlManager.newPlugin(HT_ACCESS_CONTROL,acl));
+            m_acl.reset(SPConfig::getConfig().AccessControlManager.newPlugin(HT_ACCESS_CONTROL,acl, deprecationSupport));
         }
         else {
             acl = XMLHelper::getFirstChildElement(e,_AccessControl);
             if (acl) {
                 log.info("building XML-based AccessControl provider...");
-                m_acl.reset(SPConfig::getConfig().AccessControlManager.newPlugin(XML_ACCESS_CONTROL,acl));
+                m_acl.reset(SPConfig::getConfig().AccessControlManager.newPlugin(XML_ACCESS_CONTROL,acl, deprecationSupport));
             }
             else {
                 acl = XMLHelper::getFirstChildElement(e,AccessControlProvider);
@@ -204,7 +200,7 @@ void Override::loadACL(const DOMElement* e, Category& log)
                     string t(XMLHelper::getAttrString(acl, nullptr, _type));
                     if (!t.empty()) {
                         log.info("building AccessControl provider of type %s...", t.c_str());
-                        m_acl.reset(SPConfig::getConfig().AccessControlManager.newPlugin(t.c_str(), acl));
+                        m_acl.reset(SPConfig::getConfig().AccessControlManager.newPlugin(t.c_str(), acl, deprecationSupport));
                     }
                     else {
                         throw ConfigurationException("<AccessControlProvider> missing type attribute.");
@@ -213,7 +209,7 @@ void Override::loadACL(const DOMElement* e, Category& log)
             }
         }
     }
-    catch (std::exception& ex) {
+    catch (const std::exception& ex) {
         log.crit("exception building AccessControl provider: %s", ex.what());
         m_acl.reset(new AccessControlDummy());
     }
@@ -223,7 +219,8 @@ Override::Override(bool unicodeAware, const DOMElement* e, Category& log, const
     : m_unicodeAware(unicodeAware)
 {
     // Load the property set.
-    load(e, nullptr, this);
+    xmltooling::QName unsetter(nullptr, "unset");
+    load(e, nullptr, this, nullptr, &unsetter);
     setParent(base);
 
     // Load any AccessControl provider.
@@ -261,7 +258,7 @@ Override::Override(bool unicodeAware, const DOMElement* e, Category& log, const
 
             if (*n) {
                 // Create a placeholder Path element for the first path segment and replant under it.
-                DOMElement* newpath = path->getOwnerDocument()->createElementNS(shibspconstants::SHIB2SPCONFIG_NS, Path);
+                DOMElement* newpath = path->getOwnerDocument()->createElementNS(path->getNamespaceURI(), Path);
                 newpath->setAttributeNS(nullptr, name, namebuf.c_str());
                 path->setAttributeNS(nullptr, name, n);
                 path->getParentNode()->replaceChild(newpath, path);
@@ -298,7 +295,7 @@ Override::Override(bool unicodeAware, const DOMElement* e, Category& log, const
             }
             free(dup);
         }
-        catch (std::exception&) {
+        catch (const std::exception&) {
             free(dup);
             throw;
         }
@@ -316,12 +313,21 @@ Override::Override(bool unicodeAware, const DOMElement* e, Category& log, const
 
             boost::shared_ptr<Override> o(new Override(m_unicodeAware, path, log, this));
 
-            bool flag = XMLHelper::getAttrBool(path, true, ignoreCase);
+            bool caseSensitive;
+            if (path && path->hasAttributeNS(nullptr, ignoreCase)) {
+                // In this one case, we've left ignoreCase reversed (true means case sensitive, false means insensitive).
+                // This was to protect people who followed the security advisory for SSPCPP-691 and reversed their setting.
+                log.error("DEPRECATED: ignoreCase attribute in PathRegex element will be interpreted backwards. Replace with caseSensitive");
+                caseSensitive = XMLHelper::getAttrBool(path, true, ignoreCase);
+            } else {
+                // If the old ignoreCase setting isn't set, then we just process normally.
+                caseSensitive = XMLHelper::getCaseSensitive(path, false);
+            }
             try {
-                boost::shared_ptr<RegularExpression> re(new RegularExpression(n, flag ? &chNull : ignoreOption));
+                boost::shared_ptr<RegularExpression> re(new RegularExpression(n, caseSensitive ? &chNull : caseInsensitiveOption));
                 m_regexps.push_back(make_pair(re, o));
             }
-            catch (XMLException& ex) {
+            catch (const XMLException& ex) {
                 auto_ptr_char tmp(ex.getMessage());
                 log.error("caught exception while parsing PathRegex regular expression (%d): %s", i, tmp.get());
                 throw ConfigurationException("Invalid regular expression in PathRegex element.");
@@ -346,9 +352,9 @@ Override::Override(bool unicodeAware, const DOMElement* e, Category& log, const
         try {
             boost::shared_ptr<Override> o(new Override(m_unicodeAware, path, log, this));
             boost::shared_ptr<RegularExpression> re((v && *v) ? new RegularExpression(v) : nullptr);
-            m_queries.push_back(make_tuple(string(ntemp.get()), re, o));
+            m_queries.push_back(boost::make_tuple(string(ntemp.get()), re, o));
         }
-        catch (XMLException& ex) {
+        catch (const XMLException& ex) {
             auto_ptr_char tmp(ex.getMessage());
             log.error("caught exception while parsing Query regular expression (%d): %s", i, tmp.get());
             throw ConfigurationException("Invalid regular expression in Query element.");
@@ -404,9 +410,14 @@ const Override* Override::locate(const HTTPRequest& request) const
             path2 = path2.substr(0, sep);
 
         for (vector< pair< boost::shared_ptr<RegularExpression>,boost::shared_ptr<Override> > >::const_iterator re = o->m_regexps.begin(); re != o->m_regexps.end(); ++re) {
-            if (re->first->matches(path2.c_str())) {
-                o = re->second.get();
-                break;
+            try {
+                if (re->first->matches(path2.c_str())) {
+                    o = re->second.get();
+                    break;
+                }
+            } catch (const XMLException& ex) {
+                auto_ptr_char tmp(ex.getMessage());
+                throw ConfigurationException("Caught exception while matching PathRegex : $1", params(1, tmp.get()));
             }
         }
     }
@@ -418,16 +429,21 @@ const Override* Override::locate(const HTTPRequest& request) const
         CGIParser cgi(request, true);
         do {
             descended = false;
-            for (vector< tuple< string,boost::shared_ptr<RegularExpression>,boost::shared_ptr<Override> > >::const_iterator q = o->m_queries.begin(); !descended && q != o->m_queries.end(); ++q) {
+            for (vector< boost::tuple< string,boost::shared_ptr<RegularExpression>,boost::shared_ptr<Override> > >::const_iterator q = o->m_queries.begin(); !descended && q != o->m_queries.end(); ++q) {
                 pair<CGIParser::walker,CGIParser::walker> vals = cgi.getParameters(q->get<0>().c_str());
                 if (vals.first != vals.second) {
                     if (q->get<1>()) {
                         // We have to match one of the values.
                         while (vals.first != vals.second) {
-                            if (q->get<1>()->matches(vals.first->second)) {
-                                o = q->get<2>().get();
-                                descended = true;
-                                break;
+                            try{
+                                if (q->get<1>()->matches(vals.first->second)) {
+                                    o = q->get<2>().get();
+                                    descended = true;
+                                    break;
+                                }
+                            } catch (XMLException& ex) {
+                                auto_ptr_char tmp(ex.getMessage());
+                                throw ConfigurationException("Caught exception while matching Query regular expression : $1", params(1, tmp.get()));
                             }
                             ++vals.first;
                         }
@@ -452,11 +468,18 @@ XMLRequestMapperImpl::XMLRequestMapperImpl(const DOMElement* e, Category& log) :
 #endif
     static const XMLCh _RequestMap[] =  UNICODE_LITERAL_10(R,e,q,u,e,s,t,M,a,p);
 
-    if (e && !XMLHelper::isNodeNamed(e, SHIB2SPCONFIG_NS, _RequestMap))
+    if (e && !XMLHelper::isNodeNamed(e, shibspconstants::SHIB2SPCONFIG_NS, _RequestMap)
+          && !XMLHelper::isNodeNamed(e, shibspconstants::SHIB3SPCONFIG_NS, _RequestMap)) {
         throw ConfigurationException("XML RequestMapper requires conf:RequestMap at root of configuration.");
+    }
+
+    if (XMLString::equals(e->getNamespaceURI(), shibspconstants::SHIB2SPCONFIG_NS)) {
+        log.warn("DEPRECATED: legacy 2.0 configuration, support will be removed from a future version of the software");
+    }
 
     // Load the property set.
-    load(e, nullptr, this);
+    xmltooling::QName unsetter(nullptr, "unset");
+    load(e, nullptr, this, nullptr, &unsetter);
 
     // Inject "default" app ID if not explicit.
     if (!getString("applicationId").first)
@@ -479,14 +502,14 @@ XMLRequestMapperImpl::XMLRequestMapperImpl(const DOMElement* e, Category& log) :
 
         boost::shared_ptr<Override> o(new Override(m_unicodeAware, host, log, this));
 
-        const XMLCh* flag = host->getAttributeNS(nullptr,ignoreCase);
+        const bool caseSensitive = XMLHelper::getCaseSensitive(host, false);
         try {
             boost::shared_ptr<RegularExpression> re(
-                new RegularExpression(n, (flag && (*flag==chLatin_f || *flag==chDigit_0)) ? &chNull : ignoreOption)
+                new RegularExpression(n, caseSensitive ? &chNull : caseInsensitiveOption)
                 );
             m_regexps.push_back(make_pair(re, o));
         }
-        catch (XMLException& ex) {
+        catch (const XMLException& ex) {
             auto_ptr_char tmp(ex.getMessage());
             log.error("caught exception while parsing HostRegex regular expression (%d): %s", i, tmp.get());
         }
@@ -610,8 +633,14 @@ const Override* XMLRequestMapperImpl::findOverride(const char* vhost, const HTTP
         o = i->second.get();
     else {
         for (vector< pair< boost::shared_ptr<RegularExpression>,boost::shared_ptr<Override> > >::const_iterator re = m_regexps.begin(); !o && re != m_regexps.end(); ++re) {
-            if (re->first->matches(vhost))
-                o=re->second.get();
+            try{
+                if (re->first->matches(vhost))
+                    o=re->second.get();
+            } catch (XMLException& ex) {
+                auto_ptr_char tmp(ex.getMessage());
+                throw ConfigurationException("Caught exception while matching HostRegex : $1", params(1, tmp.get()));
+            }
+
         }
     }
 
@@ -649,7 +678,7 @@ RequestMapper::Settings XMLRequestMapper::getSettings(const HTTPRequest& request
         const Override* o = m_impl->findOverride(vhost.c_str(), request);
         return Settings(o, o->getAC());
     }
-    catch (XMLException& ex) {
+    catch (const XMLException& ex) {
         auto_ptr_char tmp(ex.getMessage());
         m_log.error("caught exception while locating content settings: %s", tmp.get());
         throw ConfigurationException("XML-based RequestMapper failed to retrieve content settings.");
